//*****************************************************************************
//
//	LCD T6963	(Anag Vision AV241282BNBW-WTV#)		C181686-62 oder C181681-62
//				NHD-240128WG-ATFH-VZ#,NHD-240128WG-BTML-VZ#  
//				www.newhavendisplay.com bei www.mouser.com)
//
//	PORTC Daten
//	PORTB Steuerltg.
//	PB0 = /RST
//	PB1 = /CS
//	PB2 = /RD
//	PB3 = /WR
//	PB4 = C/D
//	PB5 = MD2 Fontumschaltung
//	PB6 = Display Licht
//	LCD Pin3 / Pin 17 Poti fuer Kontrast
//*****************************************************************************
#include <avr/io.h>
#include <avr/wdt.h>
#include <stdio.h>
#include <stdlib.h>     
#include <string.h>     
#include <math.h>       
#include "LCD_T6963.H"

#ifdef BIGCHARS
#include "ARTEFACT.h"
#endif

unsigned int ui_linechars		= LCD_XMAX / 8;
unsigned int ui_lines			= LCD_YMAX / 8;
unsigned char uc_displaymode	= 0x80;
unsigned char uc_inversmode		= 0;
unsigned char uc_pixels         = 8;

#ifdef CLOCK
unsigned int ui_MPX,ui_MPY,ui_radius,ui_abstand;
unsigned char uc_h,uc_min,uc_sec;
#endif

#ifdef GAUGE
unsigned int ui_MPGAUGEX,ui_MPGAUGEY,ui_GAUGE_Radius;
unsigned int ui_GAUGE_Value,ui_GAUGE_Scale;
#endif

#ifdef CHART
unsigned char uc_chartmode;
unsigned char uc_chartshift;
#endif



//*****************************************************************************
//	Init
//  Input:	-
//	Output:	-
//*****************************************************************************
void LCD_INIT(void)
{
   // Port B
   PORTB.DIR = 0xFF;
   PORTB.OUT = 0x3F;

   // Port C
   PORTC.DIR = 0xFF;
   PORTC.OUT = 0x00;
   
   LCD_CS(0);
   LCD_FONT(0);
   LCD_RD(0);
   LCD_WR(0);
   LCD_CMD(0);
   LCD_RST(0);

   LCD_RST(1);							//Reset
   LCD_DELAY(1);  
   LCD_RST(0);	
   LCD_DELAY(10);
  
   LCD_WRITEDATA(G_BASE % 256);			//Graphic Speicher
   LCD_WRITEDATA(G_BASE >> 8);
   LCD_WRITECOMMAND(LCD_COMMAND_GRAPHICMEMORY);

   LCD_WRITEDATA(T_BASE % 256);			//Text Speicher
   LCD_WRITEDATA(T_BASE >> 8);
   LCD_WRITECOMMAND(LCD_COMMAND_TEXTMEMORY);

   LCD_WRITEDATA(CGRAML >> 11);			// Offset fr CGRAM
   LCD_WRITEDATA(00);
   LCD_WRITECOMMAND(LCD_COMMAND_GCRAM);				

   LCD_FONT(1);							//Font 6x8 (1) oder 8x8 (0)
   LCD_DISPLAYMODE(LCD_MODE_EXOR);		//Mode OR Mode
   
   LCD_CLS();							//Clear Screen
   LCD_CURSORMODE(1);
   LCD_CURSOR(3);
   LCD_Backlight(1);
}


//*****************************************************************************
//	RST
//  Input:	L-aktiv
//	Output:	-
//*****************************************************************************
void LCD_RST(unsigned char uc_x)
{
   if (uc_x) PORTB.OUT &= 0xFE; else PORTB.OUT |= 0x01;
}


//*****************************************************************************
//	CS
//  Input:	L-aktiv
//	Output:	-
//*****************************************************************************
void LCD_CS(unsigned char uc_x)
{
   if (uc_x) PORTB.OUT &= 0xFD; else PORTB.OUT |= 0x02;
}


//*****************************************************************************
//	RD
//  Input:	L-aktiv
//	Output:	-
//*****************************************************************************
void LCD_RD(unsigned char uc_x)
{
   if (uc_x) PORTB.OUT &= 0xFB; else PORTB.OUT |= 0x04;
}


//*****************************************************************************
//	WR
//  Input:	L-aktiv
//	Output:	-
//*****************************************************************************
void LCD_WR(unsigned char uc_x)
{
   if (uc_x) PORTB.OUT &= 0xF7; else PORTB.OUT |= 0x08;
}


//*****************************************************************************
//	CMD
//  Input:	1 = CMD, 0 = Data
//	Output:	-
//*****************************************************************************
void LCD_CMD(unsigned char uc_x)
{
   if (!uc_x) PORTB.OUT &= 0xEF; else PORTB.OUT |= 0x10;
}


//*****************************************************************************
//	FONT
//  Input:	1 = 40Char 6x8, 0 = 30Char 8x8
//	Output:	-
//*****************************************************************************
void LCD_FONT(unsigned char uc_x)
{
   if (!uc_x) 
   {
	  PORTB.OUT &= 0xDF;
	  ui_linechars = LCD_XMAX / 8;
	  ui_lines     = LCD_YMAX / 8; 
	  uc_pixels    = 8;	
   }
    else {
		    PORTB.OUT |= 0x20;
			ui_linechars = LCD_XMAX / 6;
			ui_lines     = LCD_YMAX / 8;
			uc_pixels    = 6;
	     }
		 
   LCD_WRITEDATA(ui_linechars % 256);
   LCD_WRITEDATA(ui_linechars >> 8);
   LCD_WRITECOMMAND(LCD_COMMAND_GRAPHICBYTE_PER_LINE);

   LCD_WRITEDATA(ui_linechars % 256);
   LCD_WRITEDATA(ui_linechars >> 8);
   LCD_WRITECOMMAND(LCD_COMMAND_TEXTBYTE_PER_LINE);		 
#ifdef BIGCHARS   
   LCD_CREATEARTEFACT();
#endif   
}


//*****************************************************************************
//	DisplayPixels
//  Input:	-
//	Output:	Pixels
//*****************************************************************************
unsigned char LCD_GETPIXELCOUNT(void)
{	
   return(uc_pixels);	
}


//*****************************************************************************
//	DisplayMode
//  Input:	Mode
//	Output:	-
//*****************************************************************************
void LCD_DISPLAYMODE(unsigned char uc_NewMode)
{
   uc_displaymode = uc_displaymode & 0xF0;
   uc_displaymode |= uc_NewMode & 0x07;
   LCD_WRITECOMMAND(uc_displaymode);	
}


//*****************************************************************************
//	getDisplayMode
//  Input:	-
//	Output:	Mode
//*****************************************************************************
unsigned char LCD_GETDISPLAYMODE(void)
{
   return(uc_displaymode & 0x07);
}


//*****************************************************************************
//	GCMode - Zeichengenerator aus ROM oder RAM
//  Input:	GCMode
//	Output:	-
//*****************************************************************************
void LCD_GCMODE(unsigned char uc_NewMode)
{
   uc_displaymode = uc_displaymode & 0xF7;
   uc_displaymode |= (uc_NewMode & 0x08);
   LCD_WRITECOMMAND(uc_displaymode);
}


//*****************************************************************************
//	getGCMode - Zeichengenerator aus ROM oder RAM
//  Input:	GCMode
//	Output:	-
//*****************************************************************************
unsigned char LCD_GETGCMODE(void)
{
   return(uc_displaymode & 0x08);	
}


//*****************************************************************************
//	Darstellung invers
//  Input:	0/1
//	Output:	-
//*****************************************************************************
void LCD_INVERS(unsigned char uc_x)
{
   if (uc_x) uc_inversmode = 1; else uc_inversmode = 0;	
   LCD_CLS();
}


//*****************************************************************************
//	Backlight
//  Input:	1 = an
//	Output:	-
//*****************************************************************************
void LCD_Backlight(unsigned char uc_x)
{
   if (!uc_x) PORTB.OUT &= 0xBF; else PORTB.OUT |= 0x40;
}


//*****************************************************************************
//	ui_linechars
//  Input:	-
//	Output:	n
//*****************************************************************************
unsigned int LCD_ui_linechars(void)
{
   return(ui_linechars);	
}


//*****************************************************************************
//	WRITEDATA
//  Input:	Data
//	Output:	-
//*****************************************************************************
void LCD_WRITEDATA(unsigned char uc_x)
{
   do {} while ((0x03 & LCD_GETSTATUS()) != 0x03); // wait until display ready

   LCD_CMD(0);
   LCD_RD(0);
   LCD_WR(1);
   PORTC.OUT = uc_x;

   LCD_CS(1);
   asm("nop");
   asm("nop");
   LCD_CS(0);
   LCD_WR(0);
}


//*****************************************************************************
//	WRITEDATA
//  Input:	Data
//	Output:	-
//*****************************************************************************
void LCD_WRITEBLOCKDATA(unsigned char uc_x)
{
   do {} while ((0x08 & LCD_GETSTATUS()) != 0x08); // wait until display ready

   LCD_CMD(0);
   LCD_RD(0);
   LCD_WR(1);
   PORTC.OUT = uc_x;

   LCD_CS(1);
   asm("nop");
   asm("nop");
   LCD_CS(0);
   LCD_WR(0);
}


//*****************************************************************************
//	GETDATA
//  Input:	-
//	Output:	Data
//*****************************************************************************
unsigned char LCD_READDATA(void)
{
unsigned char uc_x;

   do {} while ((0x03 & LCD_GETSTATUS()) != 0x03); // wait until display ready

   PORTC.DIR = 0x00;
   PORTC.OUT = 0xFF;
   LCD_CMD(0);
   LCD_WR(0);  
   LCD_RD(1);

   LCD_CS(1);
   asm("nop");
   asm("nop");  
   uc_x = PORTC.IN;	
   LCD_CS(0);
   LCD_RD(0);

   PORTC.OUT = 0x00;
   PORTC.DIR = 0xFF;
   return(uc_x);
}


//*****************************************************************************
//	WRITECOMMAND
//  Input:	COMMAND
//	Output:	-
//*****************************************************************************
void LCD_WRITECOMMAND(unsigned char uc_x)
{
   do {} while ((0x03 & LCD_GETSTATUS()) != 0x03); // wait until display ready

   LCD_CMD(1);
   PORTC.OUT = uc_x;
   LCD_RD(0);
   LCD_WR(1);

   LCD_CS(1);
   asm("nop");
   asm("nop");
   LCD_CS(0);

   LCD_WR(0);
   LCD_CMD(0);
}


//*****************************************************************************
//	GETSTATUS
//  Input:	-
//	Output:	Status
//*****************************************************************************
unsigned char LCD_GETSTATUS(void)
{
unsigned char uc_x;

   PORTC.DIR = 0x00;
   PORTC.OUT = 0xFF;
   LCD_CMD(1);
   LCD_RD(1);  
   LCD_WR(0);

   LCD_CS(1);
   asm("nop");
   asm("nop");  
   uc_x = PORTC.IN;	
   LCD_CS(0);

   LCD_RD(0);
   LCD_CMD(0);
   PORTC.OUT = 0x00;
   PORTC.DIR = 0xFF;
   return(uc_x);
}


//*****************************************************************************
//	WRITEADRESSE
//  Input:	ADRESSE
//	Output:	-
//*****************************************************************************
void LCD_WRITEADRESSE(unsigned int ui_x)
{
   LCD_WRITEDATA(ui_x %256);
   LCD_WRITEDATA(ui_x >> 8);
   LCD_WRITECOMMAND(LCD_COMMAND_ADDRESS);	
}


//*****************************************************************************
//	DELAY
//  Input:	wert [* 10ms]
//	Output:	-
//*****************************************************************************
void LCD_DELAY(unsigned long ul_x)
{
unsigned long ul_wait,ul_cnt;

   ul_wait = ((F_CPU) / 1e3) * ul_x * 2;

   for(ul_cnt= 0; ul_cnt < ul_wait; ul_cnt++)
   {
	  wdt_reset();
   }
}


//*****************************************************************************
//	GetPixelBYTE
//  Input:	Adresse
//	Output:	Byte
//*****************************************************************************
unsigned char LCD_GETPIXELBYTE(unsigned int ui_addr)
{
unsigned char uc_pixeldata;	

   LCD_WRITEADRESSE(ui_addr);   // set LCD addr. pointer
   LCD_WRITECOMMAND(LCD_COMMAND_PEEK);   
   uc_pixeldata = LCD_READDATA();
   return(uc_pixeldata);
}


//*****************************************************************************
//	CLS_TEXT
//  Input:	-
//	Output:	-
//*****************************************************************************
void LCD_CLS_TEXT(void)
{
unsigned int ui_x;
unsigned int ui_character;

   LCD_WRITEADRESSE(T_BASE);
   ui_character = ui_linechars * ui_lines;  //16 Zeilen

   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);
   for(ui_x = 0; ui_x < ui_character; ui_x++)
   {
	  LCD_WRITEBLOCKDATA(0x00);
   }
   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);   
LCD_DELAY(1);
}


//*****************************************************************************
//	CLS_GRAPHIC
//  Input:	-
//	Output:	-
//*****************************************************************************
void LCD_CLS_GRAPHIC(void)
{
unsigned int ui_x;
unsigned long ul_pixel;
unsigned char uc_pattern;
   
   LCD_WRITEADRESSE(G_BASE);
   if (ui_linechars > 30) ul_pixel = LCD_XMAX * LCD_YMAX / 6;  //5120
                  else ul_pixel = LCD_XMAX * LCD_YMAX / 8;  //3840
   
   if (uc_inversmode) uc_pattern = 0xFF; else uc_pattern = 0x00;

   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);  
   for(ui_x = 0; ui_x < ul_pixel; ui_x++)
   {
	  LCD_WRITEBLOCKDATA(uc_pattern);
   }	
   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);  
}


//*****************************************************************************
//	CLS
//  Input:	-
//	Output:	-
//*****************************************************************************
void LCD_CLS(void)
{
   LCD_CLS_TEXT();
   LCD_CLS_GRAPHIC();	
   LCD_CURSORPOS(1,1);
}


//*****************************************************************************
//	Pixel
//  Input:	Zeile,Spalte,Set/Clear
//	Output:	-
//*****************************************************************************
void LCD_PIXEL(unsigned int ui_x,unsigned int ui_y, unsigned char uc_x)
{
unsigned int ui_addr;       // memory address of byte containing pixel to write
unsigned int ui_xn;

   if ((ui_x >= LCD_XMAX) || (ui_y >= LCD_YMAX)) return;

   ui_addr = ui_y * ui_linechars * (unsigned int) uc_pixels + ui_x;
   ui_xn = ui_addr;
   ui_addr = ui_addr / (unsigned int) uc_pixels;
   ui_xn = ui_xn - ui_addr * (unsigned int) uc_pixels;
   ui_addr = ui_addr + G_BASE;
   LCD_WRITEADRESSE(ui_addr);   // set LCD addr. pointer

   if ((!uc_inversmode && uc_x) || (uc_inversmode && !uc_x)) 
         LCD_WRITECOMMAND(LCD_COMMAND_BITSET | (((unsigned int) uc_pixels -1) - ui_xn %8)); 
    else LCD_WRITECOMMAND(LCD_COMMAND_BITCLR | (((unsigned int) uc_pixels -1) - ui_xn %8));
}


//*****************************************************************************
//	GetPixel
//  Input:	x,y
//	Output:	0,1
//*****************************************************************************
unsigned char LCD_GETPIXEL(unsigned int ui_x, unsigned int ui_y)
{
unsigned char uc_pixeldata;	
unsigned int ui_addr;       // memory address of byte containing pixel to write
unsigned int ui_xn;

   if ((ui_x >= LCD_XMAX) || (ui_y >= LCD_YMAX)) return(0);

   ui_addr = ui_y * ui_linechars * (unsigned int) uc_pixels + ui_x;
   ui_xn = ui_addr;
   ui_addr = ui_addr / (unsigned int) uc_pixels;
   ui_xn= ui_xn - ui_addr * (unsigned int) uc_pixels;
   ui_addr = ui_addr + G_BASE;
   LCD_WRITEADRESSE(ui_addr);   // set LCD addr. pointer

   LCD_WRITECOMMAND(LCD_COMMAND_PEEK);   
   uc_pixeldata = LCD_READDATA();
   if (uc_inversmode) uc_pixeldata = uc_pixeldata ^ 0xFF;   
   return((uc_pixeldata >> (((unsigned int) uc_pixels -1) - ui_xn %8)) & 0x01);
}


//*****************************************************************************
//	SetPixel
//  Input:	x,y
//	Output:	-
//*****************************************************************************
void LCD_SETPIXEL(unsigned int ui_x, unsigned int ui_y)
{
   LCD_PIXEL(ui_x,ui_y,1);	
}


//*****************************************************************************
//	ClrPixel
//  Input:	x,y
//	Output:	-
//*****************************************************************************
void LCD_CLRPIXEL(unsigned int ui_x, unsigned int ui_y)
{
   LCD_PIXEL(ui_x,ui_y,0);	
}


//*****************************************************************************
//	BIGPixel
//  Input:	Zeile,Spalte,Set/Clear
//	Output:	-
//*****************************************************************************
void LCD_BIGPIXEL(unsigned int ui_x,unsigned int ui_y, unsigned char uc_x)
{
   LCD_PIXEL(ui_x,ui_y,uc_x & 0x01);	
   if (uc_x > 1)
    {
       LCD_PIXEL(ui_x-1,ui_y,uc_x & 0x01);   
       LCD_PIXEL(ui_x+1,ui_y,uc_x & 0x01);
       LCD_PIXEL(ui_x,ui_y-1,uc_x & 0x01);
       LCD_PIXEL(ui_x,ui_y+1,uc_x & 0x01);
	   
	   if (uc_x > 3)
	    {
		   LCD_PIXEL(ui_x-1,ui_y-1,uc_x & 0x01);
		   LCD_PIXEL(ui_x-1,ui_y+1,uc_x & 0x01);
		   LCD_PIXEL(ui_x+1,ui_y-1,uc_x & 0x01);
		   LCD_PIXEL(ui_x+1,ui_y+1,uc_x & 0x01);

 		   if (uc_x > 5)
			{
			   LCD_PIXEL(ui_x,ui_y-2,uc_x & 0x01);
			   LCD_PIXEL(ui_x,ui_y+2,uc_x & 0x01);
			   LCD_PIXEL(ui_x+2,ui_y,uc_x & 0x01);
			   LCD_PIXEL(ui_x-2,ui_y,uc_x & 0x01);
			   
			   if (uc_x > 7)
				{					
				   LCD_PIXEL(ui_x-2,ui_y-2,uc_x & 0x01);
				   LCD_PIXEL(ui_x-2,ui_y+2,uc_x & 0x01);
				   LCD_PIXEL(ui_x+2,ui_y-2,uc_x & 0x01);
				   LCD_PIXEL(ui_x+2,ui_y+2,uc_x & 0x01);	
				   LCD_PIXEL(ui_x-2,ui_y-1,uc_x & 0x01);
				   LCD_PIXEL(ui_x-1,ui_y-2,uc_x & 0x01);
				   LCD_PIXEL(ui_x-2,ui_y+1,uc_x & 0x01);
				   LCD_PIXEL(ui_x-1,ui_y+2,uc_x & 0x01);
				   LCD_PIXEL(ui_x+2,ui_y-1,uc_x & 0x01);
				   LCD_PIXEL(ui_x+1,ui_y-2,uc_x & 0x01);
				   LCD_PIXEL(ui_x+2,ui_y+1,uc_x & 0x01);
				   LCD_PIXEL(ui_x+1,ui_y+2,uc_x & 0x01);				   			   			
				   LCD_PIXEL(ui_x,ui_y-3,uc_x & 0x01);
				   LCD_PIXEL(ui_x,ui_y+3,uc_x & 0x01);
				   LCD_PIXEL(ui_x-3,ui_y,uc_x & 0x01);
				   LCD_PIXEL(ui_x+3,ui_y,uc_x & 0x01);
				}
			}		   
		}
	}
}


//*****************************************************************************
//	Cursor
//  Input:	Mode 0/1 OFF, 2 ON, 3 BLINK
//	Output:	-
//*****************************************************************************
void LCD_CURSOR(unsigned char uc_x)
{
   LCD_WRITECOMMAND(LCD_COMMAND_DISPLAYMODE | (uc_x & 0x03)); 	
}


//*****************************************************************************
//	Cursor
//  Input:	Zeilen 1..8
//	Output:	-
//*****************************************************************************
void LCD_CURSORMODE(unsigned char uc_x)
{
   if (uc_x > 0) LCD_WRITECOMMAND(LCD_COMMAND_CURSORMODE | ((uc_x-1) & 0x07)); 	
}


//*****************************************************************************
//	CursorPos
//  Input:	Spalte,Zeile
//	Output:	-
//*****************************************************************************
void LCD_CURSORPOS(unsigned int ui_x, unsigned int ui_y)
{
   if ((ui_x < 1) || (ui_x > ui_linechars) || (ui_y < 1) || (ui_y > ui_lines)) return;
   	
   LCD_WRITEDATA((ui_x-1) %256);					//Cursor Position
   LCD_WRITEDATA((ui_y-1) %256);
   LCD_WRITECOMMAND(LCD_COMMAND_SETCURSOR);   
}


//*****************************************************************************
//	WRITESTRING
//  Input:	Spalte,Zeile,String
//	Output:	-
//*****************************************************************************
void LCD_WRITESTR(unsigned int ui_x,unsigned int ui_y,char * pc_string)  
{
unsigned int ui_i;
unsigned char uc_c;
unsigned int ui_addr;

   if ((ui_x < 1) || (ui_x > ui_linechars) || (ui_y < 1) || (ui_y > ui_lines)) return;

   ui_addr = T_BASE + ((ui_y-1) * ui_linechars) + (ui_x-1);
   LCD_WRITEADRESSE(ui_addr);			// set LCD adr 

   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);
   for (ui_i = 0; pc_string[ui_i] !=0; ui_i++) 
	{
	   uc_c = pc_string[ui_i] - 0x20;      // convert ASCII to LCD char address
		if (uc_c < 0) uc_c = 0;
		LCD_WRITEBLOCKDATA(uc_c);       

		ui_x++;
		if (ui_x > ui_linechars)
		{
		   ui_x = 1;
		   ui_y++;
	    }

		if (ui_y > ui_lines) ui_y = 1;
   }
   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);
   LCD_CURSORPOS(ui_x,ui_y);
}


//*****************************************************************************
//	WRITECHAR
//  Input:	Spalte,Zeile,Char
//	Output:	-
//*****************************************************************************
void LCD_WRITECHAR(unsigned int ui_x,unsigned int ui_y,char uc_char)
{
unsigned char uc_c;
unsigned int ui_addr;

   if ((ui_x < 1) || (ui_x > ui_linechars) || (ui_y < 1) || (ui_y > ui_lines)) return;

   ui_addr = T_BASE + ((ui_y-1) * ui_linechars) + (ui_x-1);
   LCD_WRITEADRESSE(ui_addr);			// set LCD adr

	uc_c = uc_char - 0x20;				// convert ASCII to LCD char address
	if (uc_c < 0) uc_c = 0;
	LCD_WRITEDATA(uc_c);
	LCD_WRITECOMMAND(LCD_COMMAND_DATA);	// write character, increment memory ptr.

	ui_x++;
	if (ui_x > ui_linechars)
	{
	   ui_x = 1;
	   ui_y++;	
	} 

	if (ui_y > ui_lines) ui_y = 1;
	LCD_CURSORPOS(ui_x,ui_y);
}
	

//*****************************************************************************
//	DELETECHAR
//  Input:	Spalte,Zeile
//	Output:	-
//*****************************************************************************
void LCD_DELETECHAR(unsigned int ui_x,unsigned int ui_y)
{
    LCD_WRITECHAR(ui_x,ui_y,' ');
	LCD_CURSORPOS(ui_x,ui_y);		  
}


//*****************************************************************************
//	GETCHAR
//  Input:	Spalte,Zeile
//	Output:	Zeichen
//*****************************************************************************
unsigned char LCD_GETCHAR(unsigned int ui_x,unsigned int ui_y)
{
unsigned int ui_addr;

   if ((ui_x < 1) || (ui_x > ui_linechars) || (ui_y < 1) || (ui_y > ui_lines)) return(0);

   ui_addr = T_BASE + ((ui_y-1) * ui_linechars) + (ui_x-1);
   LCD_WRITEADRESSE(ui_addr);			// set LCD adr
   LCD_WRITEDATA(0x00);	
   LCD_WRITECOMMAND(LCD_COMMAND_GETDATA);	
   return(LCD_READDATA() + 0x20);
}


//*****************************************************************************
//	CLEARLINE
//  Input:	Zeile
//	Output:	-
//*****************************************************************************
void LCD_CLEARLINE(unsigned int ui_y)
{
unsigned int ui_addr;
unsigned int ui_cnt;

   if ((ui_y < 1) || (ui_y > ui_lines)) return;

   ui_addr = T_BASE + ((ui_y-1) * ui_linechars);
   LCD_WRITEADRESSE(ui_addr);			// set LCD adr

   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);	
   for (ui_cnt = 0; ui_cnt < ui_linechars; ui_cnt++)
   {
	  LCD_WRITEBLOCKDATA(0x00);
   }	
   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);   
}


//*****************************************************************************
//	CLEARLINECHARS
//  Input:	Spalte,Zeile,Anzahl
//	Output:	-
//*****************************************************************************
void LCD_CLEARLINECHARS(unsigned int ui_x, unsigned int ui_y, unsigned int ui_cnt)
{
	unsigned int ui_addr;
	unsigned int ui_maxcnt;
	unsigned int ui_i;

   if ((ui_x < 1) || (ui_x > ui_linechars) || (ui_y < 1) || (ui_y > ui_lines)) return;

   ui_addr = T_BASE + ((ui_y-1) * ui_linechars) + (ui_x-1);
   LCD_WRITEADRESSE(ui_addr);			// set LCD adr

   ui_maxcnt = ui_linechars - (ui_x -1);	
   if (ui_cnt < ui_maxcnt) ui_maxcnt = ui_cnt;

   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);  
   for(ui_i = 0; ui_i < ui_maxcnt; ui_i++)
   {
	  LCD_WRITEBLOCKDATA(0x00);
   }
   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);   
}


//*****************************************************************************
//	Zeichen erzeugen
//  Input:  Zeichenadresse 1..128, Pattern[8] 8 Byte	
//	Output:	-
//*****************************************************************************
void LCD_CREATECHAR(unsigned int ui_adr,char * pc_pattern)
{
unsigned int addr, ui_cnt;

   if (ui_adr < 1) ui_adr = 1;
   addr = CGRAMH + (ui_adr -1) * 8;	
   LCD_WRITEDATA(addr %256);
   LCD_WRITEDATA(addr >> 8);
   LCD_WRITECOMMAND(LCD_COMMAND_ADDRESS);	// Set Address Pointer

   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);
   for (ui_cnt = 0; ui_cnt < 8; ui_cnt++)	// Zeichen 8 Byte durchlaufen
   {
	  LCD_WRITEBLOCKDATA(pc_pattern[ui_cnt]);	// Zeichen-Byte schreiben
   }
   LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);	// Schreiben und Adresszeiger inkr.
}


#ifdef GRAPHIC
//*****************************************************************************
//	Linie zeichnen
//  Input:  x1,y1,x2,y2,show
//	Output:	-
//*****************************************************************************
void LCD_LINE(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_show)
{
int i_dy;
int i_dx;
int i_stepx, i_stepy, i_fraction;

   i_dy = i_y2 - i_y1;
   i_dx = i_x2 - i_x1;
   
   if (i_dy < 0)
   {
	  i_dy = -i_dy;
	  i_stepy = -1;
   }
   else i_stepy = 1;
   		 
   if (i_dx < 0)
   {
	  i_dx = -i_dx;
	  i_stepx = -1;
   }
   else i_stepx = 1;
	     
   i_dy <<= 1;
   i_dx <<= 1;

   if ((i_x1 >= 0) && (i_x1 <= LCD_XMAX) && (i_y1 >= 0) && (i_y1 <= LCD_YMAX))	
		LCD_BIGPIXEL(i_x1,i_y1,uc_show);

   if (i_dx > i_dy)
   {
	  i_fraction = i_dy - (i_dx >> 1);
	  while (i_x1 != i_x2)
	  {
		 if (i_fraction >= 0)
		 {
			i_y1 += i_stepy;
			i_fraction -= i_dx;
		 }
		 i_x1 += i_stepx;
		 i_fraction += i_dy;
		 if ((i_x1 >= 0) && (i_x1 <= LCD_XMAX) && (i_y1 >= 0) && (i_y1 <= LCD_YMAX))
			  LCD_BIGPIXEL(i_x1,i_y1,uc_show);
	  }
   }
   else {
		   i_fraction = i_dx - (i_dy >> 1);
		   while (i_y1 != i_y2)
		   {
			  if (i_fraction >= 0)
			  {
				 i_x1 += i_stepx;
				 i_fraction -= i_dy;
			  }
			  i_y1 += i_stepy;
			  i_fraction += i_dx;
			  if ((i_x1 >= 0) && (i_x1 <= LCD_XMAX) && (i_y1 >= 0) && (i_y1 <= LCD_YMAX))
				   LCD_BIGPIXEL(i_x1,i_y1,uc_show);
		 }
	}
}


//*****************************************************************************
//	Kreis zeichnen
//  Input:  x,y,radius,show
//	Output:	-
//*****************************************************************************
void LCD_CIRCLE(int i_x, int i_y, int i_radius, unsigned char uc_show)
{
   LCD_CIRCLESEGMENT(i_x,i_y,i_radius,0xFF,uc_show);  //Vollkreis	 
}


//*****************************************************************************
//	Viertelkreis zeichnen
//  Input:  x,y,radius,show
//	Output:	-
//*****************************************************************************
void LCD_CIRCLESEGMENT(int i_x, int i_y, int i_radius, unsigned char uc_cpos, unsigned char uc_show)
{
int i_xc = 0;
int i_yc;
int i_p;
int i_xn,i_yn;

   i_yc = i_radius;
   i_p = 3 - (i_radius << 1);

   while (i_xc <= i_yc)
   {
	  if (uc_cpos & 0x01) 
	   {
		  i_xn = i_x + i_xc;
		  i_yn = i_y - i_yc;
		  if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX)) 
			LCD_BIGPIXEL(i_xn,i_yn,uc_show);
	   }
	   
	  if (uc_cpos & 0x02) 
	   {
		   i_xn = i_x + i_yc;
		   i_yn = i_y - i_xc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX)) 
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);	   
	   }
  
	  if (uc_cpos & 0x04)
	   {
		   i_xn = i_x + i_yc;
		   i_yn = i_y + i_xc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX))
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);	   
	   }
	   
	  if (uc_cpos & 0x08) 
	   {
		   i_xn = i_x + i_xc;
		   i_yn = i_y + i_yc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX))
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);
	   }
	   
	  if (uc_cpos & 0x10) 
	   {
		   i_xn = i_x - i_xc;
		   i_yn = i_y + i_yc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX))
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);
	   }
	   
	  if (uc_cpos & 0x20) 
	   {
		   i_xn = i_x - i_yc;
		   i_yn = i_y + i_xc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX))
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);
	   }
	   
	  if (uc_cpos & 0x40) 
	   {
		   i_xn = i_x - i_yc;
		   i_yn = i_y - i_xc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX))
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);
	   }
	   
	  if (uc_cpos & 0x80) 
	   {
		   i_xn = i_x - i_xc;
		   i_yn = i_y - i_yc;
		   if ((i_xn >= 0) && (i_xn <= LCD_XMAX) && (i_yn >= 0) && (i_yn <= LCD_YMAX))
			 LCD_BIGPIXEL(i_xn,i_yn,uc_show);
	   }

	  if (i_p < 0) i_p += (i_xc++ << 2) + 6;
		else i_p += ((i_xc++ - i_yc--) <<2) + 10;
   }
}


//*****************************************************************************
//	Pie
//  Input:  x,y,radius,show
//	Output:	-
//*****************************************************************************
void LCD_PIE(int i_x, int i_y, int i_radius, unsigned char uc_cpos, unsigned char uc_show)
{
int i_r;

   for (i_r = 0; i_r < i_radius; i_r++)
   {
      LCD_CIRCLESEGMENT(i_x,  i_y,  i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x+1,i_y+1,i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x-1,i_y+1,i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x+1,i_y-1,i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x-1,i_y-1,i_r,uc_cpos,uc_show);
   }  
 
   LCD_CIRCLESEGMENT(i_x,i_y,i_radius,uc_cpos,uc_show);   
}


//*****************************************************************************
//	SLICE
//  Input:  x,y,radius1,radius2,show
//	Output:	-
//*****************************************************************************
void LCD_SLICE(int i_x, int i_y, int i_radius1, int i_radius2, unsigned char uc_cpos, 
				unsigned char uc_show)
{
int i_r;

   if (i_radius1 > i_radius2)
    {
	   i_r = i_radius1;
	   i_radius1 = i_radius2;
	   i_radius2 = i_r;	
	}

   for (i_r = i_radius1; i_r < i_radius2; i_r++)
   {
      LCD_CIRCLESEGMENT(i_x,  i_y,  i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x+1,i_y+1,i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x-1,i_y+1,i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x+1,i_y-1,i_r,uc_cpos,uc_show);
      LCD_CIRCLESEGMENT(i_x-1,i_y-1,i_r,uc_cpos,uc_show);
   }  
 
   LCD_CIRCLESEGMENT(i_x,i_y,i_radius2,uc_cpos,uc_show);   
}


//*****************************************************************************
//	Rechteck zeichnen
//  Input:  x1,y1,x2,y2,show
//	Output:	-
//*****************************************************************************
void LCD_RECT(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_show)
{
   LCD_LINE(i_x1,i_y1,i_x2,i_y1,uc_show);  // up
   LCD_LINE(i_x1,i_y2,i_x2,i_y2,uc_show);  // down
   LCD_LINE(i_x2,i_y1,i_x2,i_y2,uc_show);  // right
   LCD_LINE(i_x1,i_y1,i_x1,i_y2,uc_show);  // left
}


//*****************************************************************************
//	Rechteck zeichnen
//  Input:  x1,y1,x2,y2,show
//	Output:	-
//*****************************************************************************
void LCD_FILLRECT(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_show)
{
int i_n;

   if ((i_y1 < 0) && (i_y1 > LCD_YMAX) && (i_y2 < 0) && (i_y2 > LCD_YMAX) &&
       (i_x1 < 0) && (i_x1 > LCD_XMAX) && (i_x2 < 0) && (i_x2 > LCD_XMAX)) return;

   if (i_y1 > i_y2) 
    {
	   i_n  = i_y1;
	   i_y1 = i_y2;
	   i_y2 = i_n;
	}

   if (i_x1 > i_x2)
   {
	   i_n  = i_x1;
	   i_x1 = i_x2;
	   i_x2 = i_n;
   }

   for (i_n = i_y1; i_n <= i_y2; i_n++)
   {
//      LCD_LINE(i_x1,i_n,i_x2,i_n,uc_show); 
	  LCD_HORIZ_LINE(i_x1,i_n,i_x2-i_x1+1,uc_show);
   }
}


//*****************************************************************************
//	Linie im Winkel zeichnen
//  Input:  x,y,w,ri,ro,show
//	Output:	-
//*****************************************************************************
void LCD_DEGLINE(int i_x, int i_y, int i_degree, int i_inner_radius, int i_outer_radius, 
                    unsigned char uc_show)
{
int i_fx,i_fy,i_tx,i_ty;

   i_fx = i_x + (i_inner_radius * sin(i_degree * 3.14 / 180));
   i_fy = i_y - (i_inner_radius * cos(i_degree * 3.14 / 180));
   i_tx = i_x + (i_outer_radius * sin(i_degree * 3.14 / 180));
   i_ty = i_y - (i_outer_radius * cos(i_degree * 3.14 / 180));

   LCD_LINE(i_fx,i_fy,i_tx,i_ty,uc_show);
   if (uc_show > 15)
    {
	   LCD_LINE(i_fx+1,i_fy+1,i_tx+1,i_ty+1,uc_show);
	   LCD_LINE(i_fx-1,i_fy-1,i_tx-1,i_ty-1,uc_show);
	}	
}


//*****************************************************************************
//	Prozentanzeige Box
//  Input:  x1,y1,x2,y2,percent
//	Output:	-
//*****************************************************************************
void LCD_PERCENTBOX(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_percent)
{
unsigned char uc_lcdmode;	
int i_n,i_M,i_horizon_line;
char c_str[10];
 	
   if (uc_percent > 100) return;
   uc_lcdmode = LCD_GETDISPLAYMODE(); //alte Einstellung merken

   if (i_x2 < i_x1) 
    {
	   i_n  = i_x1;	
	   i_x1 = i_x2;
	   i_x2 = i_n; 	
	}	

   if (i_y2 < i_y1)
   {
	   i_n  = i_y1;
	   i_y1 = i_y2;
	   i_y2 = i_n;
   }
	
   LCD_LINE(i_x1,i_y1,i_x2,i_y1,1);  // up
   LCD_LINE(i_x1,i_y2,i_x2,i_y2,1);  // down
   LCD_LINE(i_x2,i_y1,i_x2,i_y2,1);  // right
   LCD_LINE(i_x1,i_y1,i_x1,i_y2,1);  // left
	
   i_M = 10000 / abs(i_y2 - i_y1);

   i_horizon_line = (uc_percent *100 / i_M);
   if (i_horizon_line > 3) i_horizon_line = i_horizon_line -3;

   LCD_FILLRECT(i_x1+2,i_y2-2,i_x2-2,i_y2-2-i_horizon_line,1);   
   LCD_FILLRECT(i_x1+2,i_y1+1,i_x2-2,i_y2-i_horizon_line-2,0);

   LCD_DISPLAYMODE(LCD_MODE_EXOR);
   sprintf(c_str,"%02d%% ",uc_percent);
   LCD_WRITESTR((((i_x2 - i_x1)/2) + i_x1)/ (unsigned int) uc_pixels -1, 
				(((i_y2 - i_y1)/2) + i_y1)/8 +1, c_str);
   LCD_DISPLAYMODE(uc_lcdmode);
}


//*****************************************************************************
//	Fortschrittsanzeige Box
//  Input:  x1,y1,x2,y2,percent
//	Output:	-
//*****************************************************************************
void LCD_PROGRESSBAR(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_percent)
{
unsigned char uc_lcdmode;	
int i_n,i_M,i_vertical_line; 
char c_str[10];
 	
   if (uc_percent > 100) return;
   uc_lcdmode = LCD_GETDISPLAYMODE(); //alte Einstellung merken

   if (i_x2 < i_x1) 
    {
	   i_n  = i_x1;	
	   i_x1 = i_x2;
	   i_x2 = i_n; 	
	}	

   if (i_y2 < i_y1)
   {
	   i_n  = i_y1;
	   i_y1 = i_y2;
	   i_y2 = i_n;
   }
	
   LCD_LINE(i_x1,i_y1,i_x2,i_y1,1);  // up
   LCD_LINE(i_x1,i_y2,i_x2,i_y2,1);  // down
   LCD_LINE(i_x2,i_y1,i_x2,i_y2,1);  // right
   LCD_LINE(i_x1,i_y1,i_x1,i_y2,1);  // left
	
   i_M = 10000 / abs(i_x2 - i_x1);

   i_vertical_line = (uc_percent *100 / i_M);
   if (i_vertical_line > 3) i_vertical_line = i_vertical_line -3;

   LCD_FILLRECT(i_x1+2,i_y1+2,i_x1+2+i_vertical_line,i_y2-2,1);  
   LCD_FILLRECT(i_x1+2+i_vertical_line, i_y1+2, i_x2-1, i_y2-2, 0);

   LCD_DISPLAYMODE(LCD_MODE_EXOR);
   sprintf(c_str,"%02d%% ",uc_percent);
   LCD_WRITESTR((((i_x2 - i_x1) /2) + i_x1)/ (unsigned int) uc_pixels -1,
				(((i_y2 - i_y1) /2) + i_y1) /8 +1, c_str);
   LCD_DISPLAYMODE(uc_lcdmode);
}


//*****************************************************************************
//	Text Box
//  Input:  x1,y1,x2,y2,str
//	Output:	-
//*****************************************************************************
void LCD_TEXTBOX(int i_x1, int i_y1, int i_x2, int i_y2, char * pc_str)
{
unsigned char uc_lcdmode;
int i_cnt;

   uc_lcdmode = LCD_GETDISPLAYMODE(); //alte Einstellung merken

   if (i_x2 < i_x1)
    {
 	   i_cnt = i_x1;
	   i_x1 = i_x2;
	   i_x2 = i_cnt;
    }

   if (i_y2 < i_y1)
    {
	   i_cnt = i_y1;
	   i_y1 = i_y2;
	   i_y2 = i_cnt;
    }

   LCD_FILLRECT(i_x1,i_y1,i_x2,i_y2,1);	
	
   i_cnt = strlen(pc_str);
   LCD_DISPLAYMODE(LCD_MODE_EXOR);
   LCD_WRITESTR((((i_x2 - i_x1) /2) + i_x1) / (unsigned int) uc_pixels - i_cnt/2 +1, 
				(((i_y2 - i_y1) /2) + i_y1)/8 +1, pc_str);
   LCD_DISPLAYMODE(uc_lcdmode);	
}


//*****************************************************************************
//	Text Box
//  Input:  x Spalte,y Zeile,length Laenge,pc_str String  
//	Output:	-
//*****************************************************************************
void LCD_TEXTBOX2(unsigned int ui_x, unsigned int ui_y, unsigned int ui_length, 
					char * pc_str)
{
unsigned char uc_lcdmode;
unsigned int ui_cnt;

   uc_lcdmode = LCD_GETDISPLAYMODE(); //alte Einstellung merken

   if (ui_length == 0) ui_cnt = strlen(pc_str) +1;
                  else ui_cnt = ui_length +1;

   LCD_FILLRECT((ui_x -1) * uc_pixels,(ui_y-1) *8,(ui_x + ui_cnt) * uc_pixels,(ui_y+2) *8,1);	

   *(pc_str + ui_cnt -1) = '\0';
	
   LCD_DISPLAYMODE(LCD_MODE_EXOR);   
   LCD_WRITESTR(ui_x+1,ui_y+1,pc_str);
   LCD_DISPLAYMODE(uc_lcdmode);	
}


//*****************************************************************************
//	Text Box mit Rahmen
//  Input:  x1,y1,x2,y2,str
//	Output:	-
//*****************************************************************************
void LCD_TEXTFRAMEBOX(int i_x1, int i_y1, int i_x2, int i_y2, char * pc_str)
{
int i_cnt;

   if (i_x2 < i_x1)
   {
	   i_cnt = i_x1;
	   i_x1 = i_x2;
	   i_x2 = i_cnt;
   }

   if (i_y2 < i_y1)
   {
	   i_cnt = i_y1;
	   i_y1 = i_y2;
	   i_y2 = i_cnt;
   }

   LCD_RECT(i_x1,i_y1,i_x2,i_y2,1);
   LCD_TEXTBOX(i_x1+2,i_y1+2,i_x2-2,i_y2-2,pc_str);	
}


//*****************************************************************************
//	Text Box mit Rahmen
//  Input:  x Spalte,y Zeile,length Laenge,pc_str String
//	Output:	-
//*****************************************************************************
void LCD_TEXTFRAMEBOX2(unsigned int ui_x, unsigned int ui_y, unsigned int ui_length, 
						char * pc_str)
{
unsigned int ui_cnt;

   if (ui_length == 0) ui_cnt = strlen(pc_str) +1;
				  else ui_cnt = ui_length +1;

   LCD_RECT((ui_x -1) * uc_pixels,(ui_y-1) *8,(ui_x + ui_cnt) * uc_pixels,(ui_y+2) *8,1);

   *(pc_str + ui_cnt -1) = '\0';
   LCD_WRITESTR(ui_x+1,ui_y+1,pc_str);
}


//*****************************************************************************
//	Bitmap 
//  Input:  pBMP[], PosX, PosY, DimX, DimY, show 0/1
//	Output:	-
//*****************************************************************************
void LCD_BITMAP(const char * pBMP, unsigned int ui_x, unsigned int ui_y, 
                unsigned int ui_DIMX, unsigned int ui_DIMY,unsigned char uc_show)
{
unsigned int ui_addr;       
unsigned int ui_xnl, ui_xnr, ui_line, ui_col, ui_zx, ui_zy, ui_max_X, ui_max_Y;
unsigned char uc_z, uc_maske_l1, uc_maske_l2, uc_maske_r1, uc_maske_r2;
unsigned char uc_data;
unsigned int ui_cnt, ui_bitcnt, ui_offset;
unsigned long ul_picdata; 

   if ((ui_x >= LCD_XMAX) || (ui_y >= LCD_YMAX) || (ui_DIMX == 0) || (ui_DIMY == 0)) return;

   //Adresse fuer erstes Byte
   ui_addr = ui_y * ui_linechars * (unsigned int) uc_pixels + ui_x;	//Pixeloffset
   ui_xnl = ui_addr;
   ui_addr = ui_addr / (unsigned int) uc_pixels;					//n-tes Byte
   ui_xnl = ui_xnl - ui_addr * (unsigned int) uc_pixels;			//Bitoffset links

   ui_line = ui_addr / ui_linechars;								//Start Zeile
   ui_col = ui_addr - ui_line * ui_linechars;						//Start Spalte

   ui_max_X = ui_x + ui_DIMX;										//X -Bits
   if (ui_max_X >= LCD_XMAX) ui_max_X = LCD_XMAX;			

   ui_max_Y = ui_y + ui_DIMY;										//Y -Bits
   if (ui_max_Y >= LCD_YMAX) ui_max_Y = LCD_YMAX;

   //Adresse fuer letztes Byte
   ui_addr = ui_max_Y * ui_linechars * (unsigned int) uc_pixels + ui_max_X;	//Pixeloffset
   ui_xnr = ui_addr;
   ui_addr = ui_addr / (unsigned int) uc_pixels;					//n-tes Byte
   ui_xnr = ui_xnr - ui_addr * (unsigned int) uc_pixels;			//Bitoffset

   ui_max_Y = ui_addr / ui_linechars;								//letzte Zeile
   ui_max_X = ui_addr - ui_max_Y * ui_linechars;					//letzte Spalte
   if ((ui_max_X == 0) && (ui_x + ui_DIMX >= LCD_XMAX)) ui_max_X = ui_linechars;

   uc_z = 0xFF;
   uc_maske_l1 = (uc_z << ((unsigned int) uc_pixels - ui_xnl) %256);//Bitmaske links (and)
   uc_maske_l2 = 0xFF ^ uc_maske_l1; 								//Bitmaske links (or)
   
   uc_maske_r2 = (uc_z << ((unsigned int) uc_pixels - ui_xnr) %256);//Bitmaske rechts (or)
   uc_maske_r1 = 0xFF ^ uc_maske_r2;								//Bitmaske rechts (and)

   if (uc_pixels == 6)
    {
	   uc_maske_l1 &= 0x3F;
	   uc_maske_l2 &= 0x3F;
	   uc_maske_r1 &= 0x3F;
	   uc_maske_r2 &= 0x3F;	   	
	}   

   ui_cnt = 0;
   ui_offset = 0;
   
   //Schleife Zeilen (Y)
   for (ui_zy = ui_line; ui_zy < ui_max_Y; ui_zy++)
   {
	  ui_addr = ui_zy * ui_linechars + ui_col;
	  LCD_WRITEADRESSE(ui_addr + G_BASE);

	  ul_picdata = 0;
	  ui_bitcnt = 0;

	  //Schleife Spalten (X)
	  for (ui_zx = ui_col; ui_zx < ui_max_X +1; ui_zx++)
	  { 
		 //Daten lesen - immer 8 Bit breit
	     while ((ui_bitcnt < 16) && (ui_cnt < ui_max_X))
		 {
			ui_bitcnt += 8;
			ul_picdata |= (unsigned long) pBMP[ui_offset + ui_cnt] << (32 - ui_bitcnt);				 
	        ui_cnt++;
         }

         if (ui_zx == ui_col) //erstes Ausgabebyte
          {
	         if (ui_xnl > 0) //Datenbyte nicht mit Anzeige buendig
	          {
		        ul_picdata = ul_picdata >> ui_xnl;
		        ui_bitcnt += ui_xnl;
	          }
		  }
		  
		 uc_data = (unsigned char) ((unsigned long) ul_picdata >> (32-uc_pixels));	 
		 if (uc_show == 0) uc_data = 0;
		 if (uc_show == 2) uc_data ^= 0xFF;

         ul_picdata = ul_picdata << (unsigned int) uc_pixels;
         ui_bitcnt -= (unsigned int) uc_pixels;

		 //erstes Byte mit Daten verknuepfen		
		 if (ui_zx == ui_col)
		  {
			 ui_addr = ui_zy * ui_linechars + ui_col;
			 uc_z = LCD_GETPIXELBYTE(ui_addr + G_BASE);	//Bildhintergrund lesen 
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			 uc_z &= uc_maske_l1;						//zu ueberschreibende Bits ausmaskieren
			 uc_z |= (uc_data & uc_maske_l2);			//Daten maskieren und mit Hintergrund verknuepfen
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;	
			 LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);
			 LCD_WRITEBLOCKDATA(uc_z);
		  }
 
		 //letztes Byte mit Daten verknuepfen
		 if (ui_zx == ui_max_X)
		  {
			 LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND); 
			 
			 ui_addr = ui_zy * ui_linechars + ui_zx;
			 uc_z = LCD_GETPIXELBYTE(ui_addr + G_BASE);	//Bildhintergrund lesen
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			 uc_z &= uc_maske_r1;						//zu ueberschreibende Bits ausmaskieren
			 uc_z |=  (uc_data & uc_maske_r2);			//Daten maskieren und mit Hintergrund verknuepfen
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;			 
			 LCD_WRITEDATA(uc_z);
  		     LCD_WRITECOMMAND(LCD_COMMAND_DATA);
		  }

		 //ganze Bytes schreiben		  
		 if ((ui_zx > ui_col) && (ui_zx < ui_max_X))
		  {
			 if (uc_inversmode) uc_z = uc_data ^ 0xFF; else uc_z = uc_data;
			 LCD_WRITEDATA(uc_z);  
		  }			  
	  }
	  
	 ui_offset += ui_DIMX / 8;							//BMP Array zeilenweise a 8 Bit auslesen
	 ui_cnt = 0; 
   }
}


//*****************************************************************************
//	PatternRect 
//  Input:  pPattern[8], PosX, PosY, DimX, DimY, show 0/1
//	Output:	-
//*****************************************************************************
void LCD_PATTERNRECT(char * pPattern, unsigned int ui_x, unsigned int ui_y, 
                unsigned int ui_DIMX, unsigned int ui_DIMY,unsigned char uc_show)
{
unsigned int ui_addr;       
unsigned int ui_xnl, ui_xnr, ui_line, ui_col, ui_zx, ui_zy, ui_max_X, ui_max_Y;
unsigned char uc_z, uc_maske_l1, uc_maske_l2, uc_maske_r1, uc_maske_r2;
unsigned char uc_data;
unsigned int ui_cnt, ui_bitcnt;
unsigned long ul_picdata; 

   if ((ui_x >= LCD_XMAX) || (ui_y >= LCD_YMAX) || (ui_DIMX == 0) || (ui_DIMY == 0) || (ui_DIMX < 8)) return;

   //Adresse fuer erstes Byte
   ui_addr = ui_y * ui_linechars * (unsigned int) uc_pixels + ui_x;	//Pixeloffset
   ui_xnl = ui_addr;
   ui_addr = ui_addr / (unsigned int) uc_pixels;					//n-tes Byte
   ui_xnl = ui_xnl - ui_addr * (unsigned int) uc_pixels;			//Bitoffset links

   ui_line = ui_addr / ui_linechars;								//Start Zeile
   ui_col = ui_addr - ui_line * ui_linechars;						//Start Spalte

   ui_max_X = ui_x + ui_DIMX;										//X -Bits
   if (ui_max_X >= LCD_XMAX) ui_max_X = LCD_XMAX;			

   ui_max_Y = ui_y + ui_DIMY;										//Y -Bits
   if (ui_max_Y >= LCD_YMAX) ui_max_Y = LCD_YMAX;

   //Adresse fuer letztes Byte
   ui_addr = ui_max_Y * ui_linechars * (unsigned int) uc_pixels + ui_max_X;	//Pixeloffset
   ui_xnr = ui_addr;
   ui_addr = ui_addr / (unsigned int) uc_pixels;					//n-tes Byte
   ui_xnr = ui_xnr - ui_addr * (unsigned int) uc_pixels;			//Bitoffset

   ui_max_Y = ui_addr / ui_linechars;								//letzte Zeile
   ui_max_X = ui_addr - ui_max_Y * ui_linechars;					//letzte Spalte
	if ((ui_max_X == 0) && (ui_x + ui_DIMX >= LCD_XMAX)) ui_max_X = ui_linechars;
   
   uc_z = 0xFF;
   uc_maske_l1 = (uc_z << ((unsigned int) uc_pixels - ui_xnl) %256);//Bitmaske links (and)
   uc_maske_l2 = 0xFF ^ uc_maske_l1; 								//Bitmaske links (or)
   
   uc_maske_r2 = (uc_z << ((unsigned int) uc_pixels - ui_xnr) %256);//Bitmaske rechts (or)
   uc_maske_r1 = 0xFF ^ uc_maske_r2;								//Bitmaske rechts (and)

   if (uc_pixels == 6)
    {
	   uc_maske_l1 &= 0x3F;
	   uc_maske_l2 &= 0x3F;
	   uc_maske_r1 &= 0x3F;
	   uc_maske_r2 &= 0x3F;	   	
	}   

   ui_cnt = 0;  
   //Schleife Zeilen (Y)
   for (ui_zy = ui_line; ui_zy < ui_max_Y; ui_zy++)
   {
	  ui_addr = ui_zy * ui_linechars + ui_col;
	  LCD_WRITEADRESSE(ui_addr + G_BASE);

	  ul_picdata = 0;
	  ui_bitcnt = 0;

	  //Schleife Spalten (X)
	  for (ui_zx = ui_col; ui_zx < ui_max_X +1; ui_zx++)
	  { 
		 //Daten lesen - immer 8 Bit breit
	     while (ui_bitcnt < 16) 
		 {
			ui_bitcnt += 8;
			ul_picdata |= (unsigned long) pPattern[ui_cnt] << (32 - ui_bitcnt);				 
         }

         if (ui_zx == ui_col) //erstes Ausgabebyte
          {
	         if (ui_xnl > 0) //Datenbyte nicht mit Anzeige buendig
	          {
		        ul_picdata = ul_picdata >> ui_xnl;
		        ui_bitcnt += ui_xnl;
	          }
		  }
		  
		 uc_data = (unsigned char) ((unsigned long) ul_picdata >> (32-uc_pixels));	 
		 if (uc_show == 0) uc_data = 0;
		 if (uc_show == 2) uc_data ^= 0xFF;

         ul_picdata = ul_picdata << (unsigned int) uc_pixels;
         ui_bitcnt -= (unsigned int) uc_pixels;

		 //erstes Byte mit Daten verknuepfen		
		 if (ui_zx == ui_col)
		  {
			 ui_addr = ui_zy * ui_linechars + ui_col;
			 uc_z = LCD_GETPIXELBYTE(ui_addr + G_BASE);	//Bildhintergrund lesen 
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			 uc_z &= uc_maske_l1;						//zu ueberschreibende Bits ausmaskieren
			 uc_z |= (uc_data & uc_maske_l2);			//Daten maskieren und mit Hintergrund verknuepfen
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;	
			 LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);
			 LCD_WRITEBLOCKDATA(uc_z);
		  }
 
		 //letztes Byte mit Daten verknuepfen
		 if (ui_zx == ui_max_X)
		  {
			 LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND); 
			 
			 ui_addr = ui_zy * ui_linechars + ui_zx;
			 uc_z = LCD_GETPIXELBYTE(ui_addr + G_BASE);	//Bildhintergrund lesen
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			 uc_z &= uc_maske_r1;						//zu ueberschreibende Bits ausmaskieren
			 uc_z |=  (uc_data & uc_maske_r2);			//Daten maskieren und mit Hintergrund verknuepfen
			 if (uc_inversmode) uc_z = uc_z ^ 0xFF;			 
			 LCD_WRITEDATA(uc_z);
  		     LCD_WRITECOMMAND(LCD_COMMAND_DATA);
		  }

		 //ganze Bytes schreiben		  
		 if ((ui_zx > ui_col) && (ui_zx < ui_max_X))
		  {
			 if (uc_inversmode) uc_z = uc_data ^ 0xFF; else uc_z = uc_data;
			 LCD_WRITEDATA(uc_z);  
		  }			  
	  }
	  
	 ui_cnt++; 
	 if (ui_cnt > 7) ui_cnt = 0;
   }
}


//*****************************************************************************
//	Horizontal Linie zeichnen
//  Input:  x,y,DIMX,show
//	Output:	-
//*****************************************************************************
void LCD_HORIZ_LINE(unsigned int ui_x, unsigned int ui_y, unsigned int ui_DIMX, 
					unsigned char uc_show)
{
unsigned int ui_cnt;	
	
	if ((ui_x >= LCD_XMAX) || (ui_y >= LCD_YMAX) || (ui_DIMX == 0)) return;

	for(ui_cnt = ui_x; ui_cnt < ui_x + ui_DIMX; ui_cnt++)
	{
	   if (ui_cnt <= LCD_XMAX) LCD_PIXEL(ui_cnt,ui_y,uc_show);
	}
}
	
/*	//dauert laenger als stupide Pixel setzen
void LCD_HORIZ_LINE(unsigned int ui_x, unsigned int ui_y, unsigned int ui_DIMX,
unsigned char uc_show)
{
	unsigned int ui_addr;
	unsigned int ui_xnl, ui_xnr, ui_line, ui_col, ui_zx, ui_max_X, ui_max_Y;
	unsigned char uc_z, uc_maske_l1, uc_maske_l2, uc_maske_r1, uc_maske_r2;
	unsigned char uc_data;

	if ((ui_x >= LCD_XMAX) || (ui_y >= LCD_YMAX) || (ui_DIMX == 0)) return;
 
	//Adresse fuer erstes Byte
	ui_addr = ui_y * ui_linechars * (unsigned int) uc_pixels + ui_x;	//Pixeloffset
	ui_xnl = ui_addr;
	ui_addr = ui_addr / (unsigned int) uc_pixels;					//n-tes Byte
	ui_xnl = ui_xnl - ui_addr * (unsigned int) uc_pixels;			//Bitoffset links

	ui_line = ui_addr / ui_linechars;								//Start Zeile
	ui_col = ui_addr - ui_line * ui_linechars;						//Start Spalte

	ui_max_X = ui_x + ui_DIMX;										//X -Bits
	if (ui_max_X > LCD_XMAX) ui_max_X = LCD_XMAX;

	ui_max_Y = ui_y;

	//Adresse fuer letztes Byte
	ui_addr = ui_y * ui_linechars * (unsigned int) uc_pixels + ui_max_X;	//Pixeloffset
	ui_xnr = ui_addr;
	ui_addr = ui_addr / (unsigned int) uc_pixels;					//n-tes Byte
	ui_xnr = ui_xnr - ui_addr * (unsigned int) uc_pixels;			//Bitoffset

	ui_max_Y = ui_addr / ui_linechars;								//letzte Zeile
	ui_max_X = ui_addr - ui_max_Y * ui_linechars;					//letzte Spalte
	if ((ui_max_X == 0) && (ui_x + ui_DIMX >= LCD_XMAX)) ui_max_X = ui_linechars;
	
	uc_z = 0xFF;
	uc_maske_l1 = (uc_z << ((unsigned int) uc_pixels - ui_xnl) %256);//Bitmaske links (and)
	uc_maske_l2 = 0xFF ^ uc_maske_l1; 								//Bitmaske links (or)
	
	uc_maske_r2 = (uc_z << ((unsigned int) uc_pixels - ui_xnr) %256);//Bitmaske rechts (or)
	uc_maske_r1 = 0xFF ^ uc_maske_r2;								//Bitmaske rechts (and)

	if (uc_pixels == 6)
	 {
		uc_maske_l1 &= 0x3F;
		uc_maske_l2 &= 0x3F;
		uc_maske_r1 &= 0x3F;
		uc_maske_r2 &= 0x3F;
	 }

	//Schleife Spalten (X)
	for (ui_zx = ui_col; ui_zx < ui_max_X +1; ui_zx++)
	{
		uc_data = 0xFF;
		if (uc_show == 0) uc_data = 0;
		if (uc_show == 2) uc_data ^= 0xFF;

		//erstes Byte mit Daten verknuepfen
		if (ui_zx == ui_col)
	 	 {
			ui_addr = ui_line * ui_linechars + ui_col;
			uc_z = LCD_GETPIXELBYTE(ui_addr + G_BASE);	//Bildhintergrund lesen
			if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			uc_z &= uc_maske_l1;						//zu ueberschreibende Bits ausmaskieren
			uc_z |= (uc_data & uc_maske_l2);			//Daten maskieren und mit Hintergrund verknuepfen
			if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			LCD_WRITECOMMAND(LCD_COMMAND_BLOCKBEGIN);
			LCD_WRITEBLOCKDATA(uc_z);
		 }
			
		//letztes Byte mit Daten verknuepfen
		if (ui_zx == ui_max_X)
		 {
			LCD_WRITECOMMAND(LCD_COMMAND_BLOCKEND);
		
			ui_addr = ui_line * ui_linechars + ui_zx;
			uc_z = LCD_GETPIXELBYTE(ui_addr + G_BASE);	//Bildhintergrund lesen
			if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			uc_z &= uc_maske_r1;						//zu ueberschreibende Bits ausmaskieren
			uc_z |=  (uc_data & uc_maske_r2);			//Daten maskieren und mit Hintergrund verknuepfen
			if (uc_inversmode) uc_z = uc_z ^ 0xFF;
			LCD_WRITEDATA(uc_z);
			LCD_WRITECOMMAND(LCD_COMMAND_DATA);
		 }

		//ganze Bytes schreiben
		if ((ui_zx > ui_col) && (ui_zx < ui_max_X))
		 {
			if (uc_inversmode) uc_z = uc_data ^ 0xFF; else uc_z = uc_data;
			LCD_WRITEDATA(uc_z);
		 }      
	}
}
*/


#ifdef CLOCK
//*****************************************************************************
//	LCD_CLOCK  Ziffernblatt zeichnen
//  Input:	Mittelpunkt X,Y,DIM
//	Output:	-
//*****************************************************************************
void LCD_CLOCK(unsigned int ui_X, unsigned int ui_Y, unsigned int ui_DIMXY)
{
unsigned int ui_cnt;
	
   uc_h   = 0;
   uc_min = 0;
   uc_sec = 0;
   ui_radius = 0;
   ui_MPX = ui_X;
   ui_MPY = ui_Y;
      
   if (ui_DIMXY >= 50)
    {
       ui_radius  = ui_DIMXY / 2;
	   ui_abstand = ui_DIMXY / 10;
	   if (ui_abstand < 6) ui_abstand = 6;
		
	   LCD_CIRCLE(ui_MPX,ui_MPY,ui_radius,1);
	   LCD_CIRCLE(ui_MPX,ui_MPY,ui_radius - ui_abstand,1);

	   LCD_LINE(ui_MPX + ui_radius - ui_abstand, ui_MPY, ui_MPX + ui_radius, ui_MPY,3);
	   LCD_LINE(ui_MPX - ui_radius, ui_MPY, ui_MPX - ui_radius + ui_abstand, ui_MPY,3);  
	   LCD_LINE(ui_MPX, ui_MPY + ui_radius - ui_abstand, ui_MPX, ui_MPY + ui_radius,3);
	   LCD_LINE(ui_MPX, ui_MPY - ui_radius,ui_MPX, ui_MPY - ui_radius + ui_abstand,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,30,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,60,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,120,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,150,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,210,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,240,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,300,ui_radius -ui_abstand,ui_radius,3);
	   LCD_DEGLINE(ui_MPX,ui_MPY,330,ui_radius -ui_abstand,ui_radius,3);

	   LCD_CIRCLESEGMENT(ui_MPX,ui_MPY,2,0xFF,1);

       if ((ui_DIMXY / 10) > 8)
	    {
   		   for (ui_cnt = 0; ui_cnt < 60; ui_cnt++)
			{
			   LCD_BIGPIXEL(ui_MPX + ((ui_radius - ui_abstand/2) * sin(ui_cnt*6 * 3.14 / 180)),
				  		    ui_MPY - ((ui_radius - ui_abstand/2) * cos(ui_cnt*6 * 3.14 / 180)),3);
			}
		}

	}
}


//*****************************************************************************
//	LCD_CLOCK  Zeiger zeichnen
//  Input:	Wert [0..60],Zeigertyp[0..2],show
//	Output:	-
//*****************************************************************************
void LCD_CLOCKHAND(unsigned char uc_x,unsigned char uc_type,unsigned char uc_show)
{
int i_deg, i_laenge;

   if (ui_radius > 0)
    {	
	   i_laenge = ui_radius - ui_abstand; 
	   if (uc_type == 1) i_laenge = i_laenge /6 *5; //Minutenzeiger
	   if (uc_type == 2) i_laenge = i_laenge /3 *2; //Stundenzeiger  
	   i_laenge = i_laenge - 2;
	   i_deg = uc_x * 6;

		//Sekundenzeiger
	   if (uc_type == 0) LCD_DEGLINE(ui_MPX,ui_MPY,i_deg,ui_abstand/2,i_laenge,uc_show);
	                else LCD_DEGLINE(ui_MPX,ui_MPY,i_deg,ui_abstand/2,i_laenge,uc_show | 2);			
	}
}


//*****************************************************************************
//	LCD_CLOCK  Zeit anzeigen
//  Input:	h,min,sec
//	Output:	-
//*****************************************************************************
void LCD_SETTIME(unsigned char uc_Hour,unsigned char uc_Minute, unsigned char uc_Secound)
{
   LCD_CLOCKHAND(uc_h * 5,2,0); //alten Zeiger lschen
   LCD_CLOCKHAND(uc_min,1,0); //alten Zeiger lschen
   LCD_CLOCKHAND(uc_sec,0,0); //alten Zeiger lschen

   uc_h = uc_Hour; 		
   LCD_CLOCKHAND(uc_h * 5,2,1); 
   uc_min = uc_Minute;
   LCD_CLOCKHAND(uc_min,1,1);
   uc_sec = uc_Secound;
   LCD_CLOCKHAND(uc_sec,0,1);
}
#endif

#ifdef GAUGE
//*****************************************************************************
//	LCD_GAUGE Analogzeigerinstrument zeichnen
//  Input:	X,Y,DIM (2x Breite,1x Hoehe),string (Einheit)
//	Output:	-
//*****************************************************************************
void LCD_GAUGE(unsigned int ui_X, unsigned int ui_Y, unsigned int ui_DIMXY,
				unsigned int ui_Scale, unsigned int ui_Ticks, char * pc_Einheit)
{
unsigned int ui_length,ui_tck,ui_cnt,ui_n,ui_pos,ui_xp,ui_yp;
char c_str[10];
	
   ui_GAUGE_Radius = 0;
   ui_GAUGE_Value = 0;
   ui_GAUGE_Scale = 0;
   if (ui_DIMXY < 90) return;
   	
   ui_MPGAUGEX    = ui_X;
   ui_MPGAUGEY    = ui_Y + ui_DIMXY/2 - 10;
   ui_GAUGE_Radius= ui_DIMXY/3*2;
   ui_length      = ui_DIMXY/24;
   if (ui_length < 8) ui_length = 8;
   
   ui_tck         = ui_Ticks +1;
   ui_GAUGE_Scale = ui_Scale;
   if (ui_GAUGE_Scale > 1000) ui_GAUGE_Scale = 1000;
 
   LCD_RECT(ui_X-ui_DIMXY/6*5,ui_Y-ui_DIMXY/2,ui_X+ui_DIMXY/6*5,ui_Y+ui_DIMXY/2,1);
   LCD_CIRCLESEGMENT(ui_MPGAUGEX,ui_MPGAUGEY,ui_GAUGE_Radius,0xC3,1);	 
   LCD_FILLRECT(ui_X-ui_GAUGE_Radius,ui_Y+ui_GAUGE_Radius-ui_DIMXY/2,ui_X+ui_GAUGE_Radius,ui_Y+ui_DIMXY/2-1,0); 
   //Skale 
   for(ui_cnt = 0;ui_cnt < ui_tck; ui_cnt++)
   {
	  ui_pos = 120*ui_cnt/ui_tck; 
      LCD_DEGLINE(ui_MPGAUGEX,ui_MPGAUGEY,300 + ui_pos,ui_GAUGE_Radius,ui_GAUGE_Radius + ui_length,1);	 
	  for(ui_n = 1;ui_n < 5; ui_n++)
	  {
         LCD_DEGLINE(ui_MPGAUGEX,ui_MPGAUGEY,300 + ui_pos + 120* ui_n/ui_tck/5, ui_GAUGE_Radius,ui_GAUGE_Radius + ui_length/2,1);		  
	  }
   }
   LCD_DEGLINE(ui_MPGAUGEX,ui_MPGAUGEY,300 + 120,ui_GAUGE_Radius,ui_GAUGE_Radius + ui_length,1);

   //Beschriftung
   for(ui_cnt = 0;ui_cnt <= ui_tck; ui_cnt++)
   {
	  ui_pos = 300+ 120*ui_cnt/ui_tck;
	  ui_xp = ui_MPGAUGEX + ((ui_GAUGE_Radius + ui_length+5) * sin(ui_pos * 3.14 / 180));
	  ui_yp = ui_MPGAUGEY - ((ui_GAUGE_Radius + ui_length+5) * cos(ui_pos * 3.14 / 180));
	  
	  ui_n = ui_Scale*ui_cnt / ui_tck;
      itoa(ui_n,c_str,10);
	  
	  ui_xp = ui_xp/uc_pixels- strlen(c_str)/2 +1;
	  ui_yp = ui_yp/8 +1;
	  LCD_WRITESTR(ui_xp,ui_yp,c_str);	
   }   
/* 
#ifdef BIGCHARS
   ui_xp = ui_MPGAUGEX / uc_pixels/2 - strlen(pc_Einheit)/2 +1;
   ui_yp = (ui_MPGAUGEY - ui_GAUGE_Radius/2) / 16;
   LCD_WRITEBIGSTR(ui_xp,ui_yp,pc_Einheit);
#else   
   ui_xp = ui_MPGAUGEX / uc_pixels - strlen(pc_Einheit) /2 +1;
   ui_yp = (ui_MPGAUGEY - ui_GAUGE_Radius/2) / 8;
   LCD_WRITESTR(ui_xp,ui_yp,pc_Einheit); 
#endif
*/
   ui_xp = ui_MPGAUGEX / uc_pixels - strlen(pc_Einheit) /2 +1;
   ui_yp = (ui_MPGAUGEY - ui_GAUGE_Radius/2) / 8;
   LCD_WRITESTR(ui_xp,ui_yp,pc_Einheit);
   
   LCD_CIRCLE(ui_MPGAUGEX,ui_MPGAUGEY,2,1);
   LCD_GAUGEVALUE(0);   
}


//*****************************************************************************
//	LCD_GAUGEVALUE  Zeiger zeichnen
//  Input:	Wert [0..60],show
//	Output:	-
//*****************************************************************************
void LCD_GAUGEVALUE(unsigned int ui_x)
{
unsigned long ul_value;

   if (ui_GAUGE_Scale > 0)
    {
       LCD_DEGLINE(ui_MPGAUGEX,ui_MPGAUGEY,300 + ui_GAUGE_Value,3,ui_GAUGE_Radius-2,0); //Zeiger loeschen

       ul_value = 120 * (unsigned long) ui_x / (unsigned long) ui_GAUGE_Scale;
       if (ul_value > 125) ul_value = 125;
  	
       LCD_DEGLINE(ui_MPGAUGEX,ui_MPGAUGEY,300 + (unsigned int) ul_value,3,ui_GAUGE_Radius-2,1);	
       ui_GAUGE_Value = (unsigned int) ul_value;
	}
}
#endif

#endif


//*****************************************************************************
//	LCD_WRITELONG
//  Input:  ul, 10/16
//	Output:	-
//*****************************************************************************
void LCD_WRITELONG(unsigned int ui_x, unsigned int ui_y, unsigned long ul_x, 
					unsigned char uc_hex)
{
char c_str[20];

   if (uc_hex == 16) itoa(ul_x,c_str,16); else itoa(ul_x,c_str,10); 
   LCD_WRITESTR(ui_x,ui_y,c_str);	
}


#ifdef BIGCHARS
//*****************************************************************************
//	LCD_CREATEARTEFACT 
//  Input:  -  Zeichen aus ARTEFACT.h 
//	Output:	-
//*****************************************************************************
void LCD_CREATEARTEFACT(void)
{
char * p_zeichen;	
unsigned int ui_cnt, ui_x;
char c_ax[8];
   
   for(ui_cnt = 0; ui_cnt < 32; ui_cnt++)
   {
	  p_zeichen = (char*) artefact + ui_cnt*8;

   	  if (uc_pixels == 6)
		{
		   for (ui_x = 0;ui_x < 8; ui_x++)
			 c_ax[ui_x] = *(p_zeichen + ui_x) << 1;
	  
		   LCD_CREATECHAR(0x20 + ui_cnt+1,(char*) &c_ax);
		   LCD_CREATECHAR(0x40 + ui_cnt+1, p_zeichen);
		}
		else {
				for (ui_x = 0;ui_x < 8; ui_x++)
				  c_ax[ui_x] = *(p_zeichen + ui_x) << 2;
  
				LCD_CREATECHAR(0x20 + ui_cnt+1,(char*) &c_ax);
				LCD_CREATECHAR(0x40 + ui_cnt+1, p_zeichen);
		     }
   }
}


//*****************************************************************************
//	LCD_WRITE ARTEFACT
//  Input:	x,y Zeichennummer 0..31
//	Output:	-
//*****************************************************************************
void LCD_WRITEARTEFACT(unsigned int ui_x, unsigned int ui_y, unsigned char uc_x)
{
   if ((ui_x > 0) && (ui_x <= ui_linechars)	&& (ui_y > 0) && (ui_y <= ui_lines))
	{  
	   if ( /*(uc_pixels == 8) &&*/ ((uc_x & 0x80) == 0x80) ) 
              LCD_WRITECHAR(ui_x,ui_y,0xC0 + (uc_x & 0x1F));
         else LCD_WRITECHAR(ui_x,ui_y,0xE0 + (uc_x & 0x1F));
	}
}


//*****************************************************************************
//	LCD_WRITE CHAR 2x2
//  Input:	x,y, Zeichen
//	Output:	-
//*****************************************************************************
void LCD_WRITEBIGCHAR(unsigned int ui_x, unsigned int ui_y, unsigned char uc_x)
{
unsigned long ul_xchar = 0;
unsigned int ui_posX,ui_posY;

   ui_posX = ui_x * 2 -1;
   ui_posY = ui_y * 2 -1;
   	
	
   if ((uc_x > 0x1F) && (uc_x < 0x41)) //Sonderzeichen und Zahlen
	{
	   ul_xchar = ul_char[uc_x - 0x20];
	   LCD_WRITEARTEFACT(ui_posX  ,ui_posY  ,ul_xchar >> 24 %256); 
	   LCD_WRITEARTEFACT(ui_posX+1,ui_posY  ,(ul_xchar >> 16 %256) | 0x80);
	   LCD_WRITEARTEFACT(ui_posX  ,ui_posY+1,ul_xchar >>  8 %256);
	   LCD_WRITEARTEFACT(ui_posX+1,ui_posY+1,(ul_xchar       %256) | 0x80);
	}

   if ((uc_x > 0x40) && (uc_x < 0x80)) //Buchstaben
	{
	   ul_xchar = ul_char[(uc_x & 0x5F)- 0x20];
	   LCD_WRITEARTEFACT(ui_posX  ,ui_posY  ,ul_xchar >> 24 %256);
	   LCD_WRITEARTEFACT(ui_posX+1,ui_posY  ,(ul_xchar >> 16 %256) | 0x80);
	   LCD_WRITEARTEFACT(ui_posX  ,ui_posY+1,ul_xchar >>  8 %256);
	   LCD_WRITEARTEFACT(ui_posX+1,ui_posY+1,(ul_xchar       %256) | 0x80);
	}				
}


//*****************************************************************************
//	LCD_WRITE STRING 2x2
//  Input:	x,y, String
//	Output:	-
//*****************************************************************************
void LCD_WRITEBIGSTR(unsigned int ui_x, unsigned int ui_y, char* pc_str)
{
unsigned int ui_n;

   if (pc_str != NULL)
    {
	   ui_n = 0;
	   while (ui_n < strlen(pc_str))	
	   {
		  LCD_WRITEBIGCHAR(ui_x + ui_n, ui_y,pc_str[ui_n]); 
		  ui_n++;
	   } //while
	}	
}

#endif


#ifdef CHART
//*****************************************************************************
//	LCD_CHART  Diagramm zeichnen
//  Input:  Anzahl Balken, Mode, Name
//	Output:	-
// 
//  Mode: Bit1   = Y %-Skale
//		  Bit2,3 = Balkenabstand, Balkenbreite
//		  Bit4   = Leerzeile unten
//		  Bit5   = frei 
//		  Bit6	 = frei
//		  Bit7	 = % Zahlen am Balken
//		  Bit8	 = untere Zeile mit Verschieben bei ShiftValue
//*****************************************************************************
void LCD_CHART(unsigned char uc_mode,char * pc_name)
{
unsigned int ui_x,ui_left,ui_bottom,ui_top,ui_abst,ui_pos,ui_z,ui_k;

   if (pc_name != NULL)
    {
       ui_x = ui_linechars / 2 - strlen(pc_name) / 2;	
       LCD_WRITESTR(ui_x,1,pc_name);
	}

   uc_chartmode = uc_mode;
   LCD_CHARTARTEFACT();
   uc_chartshift = 0;
   
   if (uc_chartmode & 0x08) ui_bottom = 1; else ui_bottom = 0;
   ui_top = 2;   

   if (uc_chartmode & 0x01)
	{
	   ui_left = 4;

	   for(ui_x = ui_top+1; ui_x < ui_lines - ui_bottom; ui_x++)
	    {
	       LCD_WRITECHAR(ui_left,ui_x,0xB3); 
	    }

	   if (uc_chartmode & 0x40) LCD_WRITESTR(1,ui_top,"[%]");
	   ui_abst = ui_lines - ui_bottom - ui_top-1; 
	   ui_abst = ui_abst * 8 / 4;
	   ui_pos = (ui_lines - ui_bottom) * 8;	

	   for(ui_x = 0; ui_x <= 4; ui_x++)
	   {
		   ui_z = ui_pos - (ui_pos / 8) *8;
		   if (ui_z > 0) ui_k = ui_pos + 8; 
		     else {
				     ui_k = ui_pos;
					 ui_z = 8;
			      }
		   
		   if (ui_z < 3)                 LCD_WRITECHAR(ui_left,ui_k/8,0xB6);
		   if ((ui_z > 2) && (ui_z < 5)) LCD_WRITECHAR(ui_left,ui_k/8,0xB5);
		   if (ui_z > 4)                 LCD_WRITECHAR(ui_left,ui_k/8,0xB4);	
	 
		   if (uc_chartmode & 0x40)
		    {  
			   switch(ui_x)
			   {
				case 0: LCD_WRITESTR(1,ui_k/8,"  0"); break;
				case 1: LCD_WRITESTR(1,ui_k/8," 25"); break;
				case 2: LCD_WRITESTR(1,ui_k/8," 50"); break;
				case 3: LCD_WRITESTR(1,ui_k/8," 75"); break;
				case 4: LCD_WRITESTR(1,ui_k/8,"100"); break;
			   } //switch
			}
		   ui_pos = ui_pos - ui_abst;
	   }  
	}
    else {
		    ui_left = 1;

			for(ui_x = ui_top; ui_x < ui_lines-ui_bottom; ui_x++)
			{
			  LCD_WRITECHAR(ui_left,ui_x,0xB0);
			}

  		    LCD_WRITECHAR(ui_left,ui_lines-ui_bottom,0xB1);
	     }

   for(ui_x = ui_left+1; ui_x <= ui_linechars; ui_x++)
   {
	  LCD_WRITECHAR(ui_x,ui_lines-ui_bottom,0xB2);
   }

}


//*****************************************************************************
//	LCD_CHART_Artefact
//  Input:  -
//	Output:	-
//*****************************************************************************
void LCD_CHARTARTEFACT(void)
{
unsigned int ui_x;
unsigned char uc_z;
char c_ax[8];

   //Balkenzeichen generieren
   for(ui_x = 0; ui_x < 8; ui_x++)
      c_ax[ui_x] = 0;

   uc_z = 0;
   switch((uc_chartmode >>1) & 0x03)
   {
    case 0: uc_z = 1; break;
    case 1: uc_z = 0; break;
    case 2: uc_z = 0; break;
    case 3: uc_z = 0; break;
   } //switch
    
   for(ui_x = 0; ui_x <= 8; ui_x++)  
   { 	   	  
      LCD_CREATECHAR(0x18 + ui_x,(char*) &c_ax);
	  c_ax[8-ui_x-1] = 0xFF >> (8-uc_pixels + uc_z);  
   }
  
   //Diagramm Artefakte 
   if (uc_pixels == 6) 
    {
	   for(ui_x = 0; ui_x < 8; ui_x++)
	      c_ax[ui_x] = 0x30;
       LCD_CREATECHAR(0x11,(char*) &c_ax);

	   for(ui_x = 0; ui_x < 8; ui_x++)
	      c_ax[ui_x] = 0x30;
	   c_ax[6] = 0x3F;
	   c_ax[7] = 0x3F;
	   LCD_CREATECHAR(0x12,(char*) &c_ax);

	   for(ui_x = 0; ui_x < 8; ui_x++)
		  c_ax[ui_x] = 0x00;
	   c_ax[6] = 0x3F;
	   c_ax[7] = 0x3F;
	   LCD_CREATECHAR(0x13,(char*) &c_ax);
	   
	   for(ui_x = 0; ui_x < 8; ui_x++)
	      c_ax[ui_x] = 0x03;
	   LCD_CREATECHAR(0x14,(char*) &c_ax);

	   for(ui_x = 0; ui_x < 8; ui_x++)
	      c_ax[ui_x] = 0x03;
	   c_ax[6] = 0x3F;
	   c_ax[7] = 0x3F;
	   LCD_CREATECHAR(0x15,(char*) &c_ax);
	   
	   for(ui_x = 0; ui_x < 8; ui_x++)
	      c_ax[ui_x] = 0x03;
	   c_ax[3] = 0x3F;	 
	   c_ax[4] = 0x3F;
	   LCD_CREATECHAR(0x16,(char*) &c_ax);

	   for(ui_x = 0; ui_x < 8; ui_x++)
	      c_ax[ui_x] = 0x03;
	   c_ax[0] = 0x3F;
	   c_ax[1] = 0x3F;
	   LCD_CREATECHAR(0x17,(char*) &c_ax);	
	}
	else {		
		    for(ui_x = 0; ui_x < 8; ui_x++)
			   c_ax[ui_x] = 0xC0;
			LCD_CREATECHAR(0x11,(char*) &c_ax);

		    for(ui_x = 0; ui_x < 8; ui_x++)
			   c_ax[ui_x] = 0xC0;
	 	    c_ax[6] = 0xFF;
	 	    c_ax[7] = 0xFF;			   
		    LCD_CREATECHAR(0x12,(char*) &c_ax);
			
		    for(ui_x = 0; ui_x < 8; ui_x++)
		       c_ax[ui_x] = 0x00;
		    c_ax[6] = 0xFF;
		    c_ax[7] = 0xFF;
		    LCD_CREATECHAR(0x13,(char*) &c_ax);		
			
   			for(ui_x = 0; ui_x < 8; ui_x++)
			   c_ax[ui_x] = 0x03;
			LCD_CREATECHAR(0x14,(char*) &c_ax);
			
	 	    for(ui_x = 0; ui_x < 8; ui_x++)
			   c_ax[ui_x] = 0x03;
			c_ax[6] = 0xFF;
			c_ax[7] = 0xFF;
			LCD_CREATECHAR(0x15,(char*) &c_ax);

	 	    for(ui_x = 0; ui_x < 8; ui_x++)
	 	       c_ax[ui_x] = 0x03;
	 	    c_ax[3] = 0xFF;
	 	    c_ax[4] = 0xFF;
	 	    LCD_CREATECHAR(0x16,(char*) &c_ax);
			
	 	    for(ui_x = 0; ui_x < 8; ui_x++)
	 	       c_ax[ui_x] = 0x03;
	 	    c_ax[0] = 0xFF;
	 	    c_ax[1] = 0xFF;
	 	    LCD_CREATECHAR(0x17,(char*) &c_ax);
  		 }    
}


//*****************************************************************************
//	LCD_CHARTVALUE  Balken zeichnen
//  Input:	Balkennr.[0..n], Wert [0..100%]
//	Output:	-
//*****************************************************************************
void LCD_CHARTVALUE(unsigned char uc_n,unsigned char uc_value)
{
unsigned int ui_maxY,ui_y,ui_z,ui_cnt,ui_pos_x,ui_bottom,ui_top;
unsigned char uc_thick,uc_k;

   ui_pos_x = 2;
   if (uc_chartmode & 0x01) ui_pos_x = 6;
   
   uc_thick = 1;
   switch((uc_chartmode >> 1) &0x03)
   {
   case 0: ui_pos_x += (unsigned int) uc_n;     uc_thick = 1; break;
   case 1: ui_pos_x += (unsigned int) uc_n * 2; uc_thick = 1; break;
   case 2: ui_pos_x += (unsigned int) uc_n * 3; uc_thick = 2; break;	   
   case 3: ui_pos_x += (unsigned int) uc_n * 4;	uc_thick = 3; break;   
   } //switch
   
   if (ui_pos_x > ui_linechars) return;

   if (uc_chartmode & 0x08) ui_bottom = 1; else ui_bottom = 0;
   ui_top = 2;

   if (uc_value <   0) uc_value = 0;
   if (uc_value > 100) uc_value = 100;
   ui_maxY = (ui_lines - ui_bottom-ui_top-1) * 8; //Pixels	
   ui_y = (ui_maxY * (unsigned int) uc_value) / 100; //Balkenhoehe 
   ui_y += 2;
   ui_z = ui_y / 8;
   
   if (ui_z > 0)
    {
	   for(ui_cnt = 0; ui_cnt < ui_z; ui_cnt++)
	   {
		  for(uc_k = 0; uc_k < uc_thick; uc_k++)
		  { 
	         LCD_WRITECHAR(ui_pos_x+ uc_k,ui_lines -ui_bottom - ui_cnt,0xBF); 
		  }
       }
	}
   	
   ui_cnt = ui_y - (ui_z * 8);	

   ui_cnt &= 0x07;
   for(uc_k = 0; uc_k < uc_thick; uc_k++)
   {
      if (ui_cnt == 0) LCD_WRITECHAR(ui_pos_x + uc_k,ui_lines-ui_bottom - ui_z,0x20); 
                  else LCD_WRITECHAR(ui_pos_x + uc_k,ui_lines-ui_bottom - ui_z,0xB8 + ui_cnt-1);
   }
   
   ui_z++;
   if ((ui_z*8) < ui_maxY) 
	{
	   for(ui_cnt = ui_z; ui_cnt <= ui_maxY/8; ui_cnt++)
	   {
		  for(uc_k = 0; uc_k < uc_thick; uc_k++)
		  {
 		     LCD_WRITECHAR(ui_pos_x + uc_k,ui_lines-ui_bottom - ui_cnt,0x20); 	
		  }
	   }
	}
}


//*****************************************************************************
//	LCD_CHARTSHIFT  Balken zeichnen
//  Input:	Wert [0..100%], Shift LR [0,1]
//	Output:	-
//*****************************************************************************
void LCD_CHARTSHIFT(unsigned char uc_value, unsigned char uc_shiftmode)
{
unsigned int ui_minX,ui_maxX,ui_k,ui_x,ui_y,ui_z,ui_bottom,ui_top;

   if (uc_value <   0) uc_value = 0;
   if (uc_value > 100) uc_value = 100;

   if (uc_chartmode & 0x08) ui_bottom = 1; else ui_bottom = 0;
   ui_top = 2;
	
   ui_minX = 2;
   if (uc_chartmode & 0x01) ui_minX = 6;
   
   ui_k = 1;
   switch((uc_chartmode >> 1) &0x03)
   {
	   case 0: ui_k = 1; break;
	   case 1: ui_k = 2; break;
	   case 2: ui_k = 3; break;
	   case 3: ui_k = 4; break;
   } //switch
   
   ui_z = 0;
   ui_maxX  = ui_minX;  
   while(ui_maxX <= ui_linechars)
   {
	  ui_maxX += ui_k;
	  ui_z++;
   }
   ui_maxX -= ui_k;
   ui_z--;

   if (uc_shiftmode == 0) //links schieben	
    {
	   uc_chartshift = 0;
	   		
	   for(ui_y = ui_top; ui_y <= ui_lines - ui_bottom; ui_y++)
	    {
	       for(ui_x = ui_minX; ui_x < ui_linechars; ui_x++)
		    {
			   if ((ui_x + ui_k) > ui_linechars) 
			    {
				   if (ui_y < (ui_lines - ui_bottom)) LCD_WRITECHAR(ui_x,ui_y,0x20);
				 	 else LCD_WRITECHAR(ui_x,ui_y,0xB2);
			    }
			    else LCD_WRITECHAR(ui_x,ui_y,LCD_GETCHAR(ui_x+ui_k,ui_y));						
		    }
	    }

	   if (uc_chartmode & 0x80) 
	    {
	       for(ui_x = ui_minX; ui_x < ui_linechars; ui_x++)
	        {
			   if ((ui_x + ui_k) > ui_linechars) LCD_WRITECHAR(ui_x,ui_lines,0x20);					
			     else LCD_WRITECHAR(ui_x,ui_lines,LCD_GETCHAR(ui_x+ui_k,ui_lines));
		    }
		}
		
	   LCD_CHARTVALUE(ui_z,uc_value);
    }
    else {  //rechts schieben
			uc_chartshift = 1;
		
			for(ui_y = ui_top; ui_y <= ui_lines - ui_bottom; ui_y++)
			 {
			   for(ui_x = ui_maxX; ui_x >= ui_minX; ui_x--)
				{
				   if ((ui_x + ui_k) < ui_minX)
					{
					   if (ui_y < (ui_lines - ui_bottom)) LCD_WRITECHAR(ui_x+ui_k,ui_y,0x20);
						 else LCD_WRITECHAR(ui_x+ui_k,ui_y,0xB2);
					}
					else LCD_WRITECHAR(ui_x+ui_k,ui_y,LCD_GETCHAR(ui_x,ui_y));			
				}
			 }

			if (uc_chartmode & 0x80)
			 {
				for(ui_x = ui_maxX; ui_x >= ui_minX; ui_x--)
				 {
				    LCD_WRITECHAR(ui_x+ui_k,ui_lines,LCD_GETCHAR(ui_x,ui_lines));
				 }
			 }
			 
            LCD_CHARTVALUE(0,uc_value);
         }
}


//*****************************************************************************
//	LCD_CHARTYWRITE  Bezeichnung Y-Achse
//  Input:	- Strings[3] an Y-Achse
//	Output:	-
//*****************************************************************************
void LCD_CHARTYWRITE(char * c_s1,char * c_s2,char * c_s3,char * c_s4,char * c_s5,char * c_s6)
{
unsigned int ui_x,ui_bottom,ui_top,ui_abst,ui_pos,ui_z,ui_k,ui_n;
char c_txt[4];

   if (uc_chartmode & 0x08) ui_bottom = 1; else ui_bottom = 0;
   ui_top = 2;

   if (uc_chartmode & 0x01)
	{
	   strcpy(c_txt,"   ");
	   ui_n = strlen(c_s1);
	   if (ui_n > 0)
	    {
	       if (ui_n > 3) ui_n = 3;
	       strncpy(c_txt + 3 - ui_n,c_s1,ui_n);
		}
 	   LCD_WRITESTR(1,ui_top,c_txt);
		
	   ui_abst = ui_lines - ui_bottom - ui_top-1;
	   ui_abst = ui_abst * 8 / 4;
	   ui_pos = (ui_lines - ui_bottom) * 8;

	   for(ui_x = 0; ui_x <= 4; ui_x++)
	    {
		   ui_z = ui_pos - (ui_pos / 8) *8;
		   if (ui_z > 0) ui_k = ui_pos + 8;
			else {
			 	    ui_k = ui_pos;
					ui_z = 8;
				 }

 		   strcpy(c_txt,"   ");
		
		   switch(ui_x)
		   {
			case 0: ui_n = strlen(c_s2);
					if (ui_n > 0)
					 {
						if (ui_n > 3) ui_n = 3;
						strncpy(c_txt + 3 - ui_n,c_s2,ui_n);
					 }
					 break;
			case 1: ui_n = strlen(c_s3);
					if (ui_n > 0)
					 {
						if (ui_n > 3) ui_n = 3;
						strncpy(c_txt + 3 - ui_n,c_s3,ui_n);
					 }
					break;
			case 2: ui_n = strlen(c_s4);
					if (ui_n > 0)
					 {
						if (ui_n > 3) ui_n = 3;
						strncpy(c_txt + 3 - ui_n,c_s4,ui_n);
					 }
					break;
			case 3: ui_n = strlen(c_s5);
					if (ui_n > 0)
					 {
						if (ui_n > 3) ui_n = 3;
						strncpy(c_txt + 3 - ui_n,c_s5,ui_n);
					 }
					break;
			case 4: ui_n = strlen(c_s6);
					if (ui_n > 0)
					 {
						if (ui_n > 3) ui_n = 3;
						strncpy(c_txt + 3 - ui_n,c_s6,ui_n);
					}
					break;
		   } //switch

		   LCD_WRITESTR(1,ui_k/8,c_txt);
		   ui_pos = ui_pos - ui_abst;
		}
	}
}


//*****************************************************************************
//	LCD_CHARTXWRITE  Bezeichnung X-Achse
//  Input:	- x-Position, String[] an X-Achse
//	Output:	-
//*****************************************************************************
void LCD_CHARTXWRITE(unsigned char uc_n, char * c_s)
{
unsigned int ui_pos_x,ui_n;
unsigned char uc_thick;
char c_txt[6];

   if (uc_chartmode & 0x01)
    {
	   ui_pos_x = 2;
	   if (uc_chartmode & 0x01) ui_pos_x = 6;

	   uc_thick = 1;
	   switch((uc_chartmode >> 1) &0x03)
	   {
		case 0: ui_pos_x += (unsigned int) uc_n;     uc_thick = 1; break;
		case 1: ui_pos_x += (unsigned int) uc_n * 2; uc_thick = 1; break;
		case 2: ui_pos_x += (unsigned int) uc_n * 3; uc_thick = 2; break;
		case 3: ui_pos_x += (unsigned int) uc_n * 4; uc_thick = 3; break;
	   } //switch

	   if (ui_pos_x > ui_linechars) return;
	   
	   strcpy(c_txt,"     ");
	   ui_n = strlen(c_s);
	   if (ui_n > 0)
		{
		   if (ui_n > uc_thick) ui_n = uc_thick;
		   strncpy(c_txt,c_s,ui_n);
		   c_txt[uc_thick] = '\0';
		}			
	   LCD_WRITESTR(ui_pos_x,ui_lines,c_txt);	 
    }
}


//*****************************************************************************
//	LCD_CHARTXFIRSTWRITE  Bezeichnung X-Achse je nach Schieberichtung
//  Input:	- String[] an X-Achse
//	Output:	-
//*****************************************************************************
void LCD_CHARTXFIRSTWRITE(char * c_s)
{
unsigned int ui_minX,ui_maxX,ui_k,ui_z;

   ui_z = 0;
   if (uc_chartshift == 0)
    {
	   ui_minX = 2;
	   if (uc_chartmode & 0x01) ui_minX = 6;

	   ui_k = 1;
	   switch((uc_chartmode >> 1) &0x03)
	   {
		case 0: ui_k = 1; break;
		case 1: ui_k = 2; break;
		case 2: ui_k = 3; break;
		case 3: ui_k = 4; break;
	   } //switch

	   ui_z = 0;
	   ui_maxX  = ui_minX;
	   while(ui_maxX <= ui_linechars)
	   {
		  ui_maxX += ui_k;
		  ui_z++;
	   }
	   ui_maxX -= ui_k;
	   ui_z--;		
	}
	
   LCD_CHARTXWRITE(ui_z,c_s);
}

#endif





