//*****************************************************************************
//
//	LCD T6963
//
//*****************************************************************************
#ifndef _LCD_H_
#define _LCD_H_

//benoetigte Bestandteile der Unit
#define BIGCHARS
#define GRAPHIC
#define CLOCK
#define GAUGE
#define CHART



#define F_CPU							 32000000UL

#define LCD_XMAX								240		// Pixel (x,y) 
#define LCD_XMIN								  0
#define LCD_YMAX								128
#define LCD_YMIN								  0

#define G_BASE								 0x0400		//0x0C00h adr graphic Speicher
#define T_BASE								 0x0000		//adr text Speicher
#define CGRAML								 0x1800		//adr CGRAM
#define CGRAMH								 CGRAML + 0x80 *8

#define LCD_MODE_OR							   0x80
#define LCD_MODE_EXOR						   0x81
#define LCD_MODE_AND						   0x83
#define LCD_MODE_TA							   0x84
#define LCD_MODE_GCRAM						   0x08
#define LCD_MODE_GCROM						   0x00

#define LCD_CURSOR_OFF						   0x9C	
#define LCD_CURSOR_ON						   0x9E
#define LCD_CURSOR_BLINK_OFF				   0x9E
#define LCD_CURSOR_BLINK_ON					   0x9F
#define LCD_DISPLAY_OFF						   0x90

#define LCD_COMMAND_SETCURSOR				   0x21
#define LCD_COMMAND_GCRAM					   0x22
#define LCD_COMMAND_ADDRESS					   0x24	
#define LCD_COMMAND_TEXTMEMORY				   0x40
#define LCD_COMMAND_TEXTBYTE_PER_LINE		   0x41
#define LCD_COMMAND_GRAPHICMEMORY			   0x42
#define LCD_COMMAND_GRAPHICBYTE_PER_LINE	   0x43
#define LCD_COMMAND_DISPLAYMODE				   0x9C
#define LCD_COMMAND_CURSORMODE				   0xA0
#define LCD_COMMAND_BLOCKBEGIN				   0xB0
#define LCD_COMMAND_BLOCKEND				   0xB2
#define LCD_COMMAND_DATA					   0xC0
#define LCD_COMMAND_GETDATA					   0xC1
#define LCD_COMMAND_PEEK					   0xE0
#define LCD_COMMAND_BITCLR					   0xF0
#define LCD_COMMAND_BITSET					   0xF8


void LCD_INIT(void);
void LCD_INVERS(unsigned char uc_x);
void LCD_FONT(unsigned char uc_x);
void LCD_DISPLAYMODE(unsigned char uc_NewMode);
unsigned char LCD_GETDISPLAYMODE(void);
void LCD_GCMODE(unsigned char uc_NewMode);
unsigned char LCD_GETGCMODE(void);
unsigned char LCD_GETPIXELCOUNT(void);

void LCD_Backlight(unsigned char uc_x);
void LCD_CLS(void);
void LCD_CLS_TEXT(void);
void LCD_CLS_GRAPHIC(void);
unsigned int LCD_LINECHARS(void);

void LCD_CURSOR(unsigned char uc_x);
void LCD_CURSORMODE(unsigned char uc_x);
void LCD_CURSORPOS(unsigned int ui_x, unsigned int ui_y);

void LCD_WRITESTR(unsigned int ui_x,unsigned int ui_y,char * pc_string);
void LCD_WRITECHAR(unsigned int ui_x,unsigned int ui_y,char uc_char);
void LCD_DELETECHAR(unsigned int ui_x,unsigned int ui_y);
unsigned char LCD_GETCHAR(unsigned int ui_x,unsigned int ui_y);
void LCD_CLEARLINE(unsigned int ui_y);
void LCD_CLEARLINECHARS(unsigned int ui_x, unsigned int ui_y, unsigned int ui_cnt);
void LCD_WRITELONG(unsigned int ui_x, unsigned int ui_y, unsigned long ul_x, unsigned char uc_hex);
void LCD_CREATECHAR(unsigned int ui_adr,char * pc_pattern);

#ifdef BIGCHARS
void LCD_CREATEARTEFACT(void);
void LCD_WRITEARTEFACT(unsigned int ui_x, unsigned int ui_y, unsigned char uc_x);
void LCD_WRITEBIGCHAR(unsigned int ui_x, unsigned int ui_y, unsigned char uc_x);
void LCD_WRITEBIGSTR(unsigned int ui_x, unsigned int ui_y, char* pc_str);
#endif

#ifdef GRAPHIC
void LCD_SETPIXEL(unsigned int ui_x, unsigned int ui_y);
void LCD_CLRPIXEL(unsigned int ui_x, unsigned int ui_y);
unsigned char LCD_GETPIXEL(unsigned int ui_x, unsigned int ui_y);
void LCD_PIXEL(unsigned int ui_x,unsigned int ui_y, unsigned char uc_x);
void LCD_BIGPIXEL(unsigned int ui_x,unsigned int ui_y, unsigned char uc_x);

void LCD_LINE(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_show);
void LCD_HORIZ_LINE(unsigned int ui_x, unsigned int ui_y, unsigned int ui_DIMX, unsigned char uc_show);
void LCD_CIRCLE(int i_x, int i_y, int i_radius, unsigned char uc_show);
void LCD_CIRCLESEGMENT(int i_x, int i_y, int i_radius, unsigned char uc_cpos, unsigned char uc_show);
void LCD_PIE(int i_x, int i_y, int i_radius, unsigned char uc_cpos, unsigned char uc_show);
void LCD_SLICE(int i_x, int i_y, int i_radius1, int i_radius2, unsigned char uc_cpos, unsigned char uc_show);
void LCD_RECT(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_show);
void LCD_FILLRECT(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_show);
void LCD_DEGLINE(int i_x, int i_y, int i_degree, int i_inner_radius, int i_outer_radius, unsigned char uc_show);

void LCD_PERCENTBOX(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_percent);
void LCD_PROGRESSBAR(int i_x1, int i_y1, int i_x2, int i_y2, unsigned char uc_percent);

void LCD_TEXTBOX(int i_x1, int i_y1, int i_x2, int i_y2, char * pc_str);
void LCD_TEXTBOX2(unsigned int ui_x, unsigned int ui_y, unsigned int ui_length, char * pc_str);
void LCD_TEXTFRAMEBOX(int i_x1, int i_y1, int i_x2, int i_y2, char * pc_str);
void LCD_TEXTFRAMEBOX2(unsigned int ui_x, unsigned int ui_y, unsigned int ui_length, char * pc_str);

void LCD_BITMAP(const char * pBMP, unsigned int ui_x, unsigned int ui_y, unsigned int ui_DIMX, unsigned int ui_DIMY, unsigned char uc_show);
void LCD_PATTERNRECT(char * pPattern, unsigned int ui_x, unsigned int ui_y, unsigned int ui_DIMX, unsigned int ui_DIMY,unsigned char uc_show);

#ifdef CLOCK
void LCD_CLOCK(unsigned int ui_X, unsigned int ui_Y, unsigned int ui_DIMXY);
void LCD_SETTIME(unsigned char uc_Hour,unsigned char uc_Minute, unsigned char uc_Secound);
void LCD_CLOCKHAND(unsigned char uc_x,unsigned char uc_type,unsigned char uc_show);
#endif

#ifdef GAUGE
void LCD_GAUGE(unsigned int ui_X, unsigned int ui_Y, unsigned int ui_DIMXY, unsigned int ui_Scale, unsigned int ui_Ticks, char * pc_Einheit);
void LCD_GAUGEVALUE(unsigned int ui_x);
#endif

#endif


#ifdef CHART
void LCD_CHART(unsigned char uc_mode,char * pc_name);
void LCD_CHARTVALUE(unsigned char uc_n,unsigned char uc_value);
void LCD_CHARTSHIFT(unsigned char uc_value, unsigned char uc_shiftmode);
void LCD_CHARTARTEFACT(void);
void LCD_CHARTYWRITE(char * c_s1,char * c_s2,char * c_s3,char * c_s4,char * c_s5,char * c_s6);
void LCD_CHARTXWRITE(unsigned char uc_n, char * c_s);
void LCD_CHARTXFIRSTWRITE(char * c_s);
#endif

//interne Fkt.
void LCD_RST(unsigned char uc_x);
void LCD_CS(unsigned char uc_x);
void LCD_RD(unsigned char uc_x);
void LCD_WR(unsigned char uc_x);
void LCD_CMD(unsigned char uc_x);
void LCD_WRITEDATA(unsigned char uc_x);
void LCD_WRITEBLOCKDATA(unsigned char uc_x);
unsigned char LCD_READDATA(void);
void LCD_WRITECOMMAND(unsigned char uc_x);
unsigned char LCD_GETSTATUS(void);
void LCD_WRITEADRESSE(unsigned int ui_x);
void LCD_DELAY(unsigned long ul_x);
void LCD_PIXEL(unsigned int ui_x,unsigned int ui_y, unsigned char uc_x);
unsigned char LCD_GETPIXELBYTE(unsigned int ui_addr);

#endif


