//****************************************************************************
//
//	Kommandointerpreter fuer UART D0
//
//*****************************************************************************
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "kommandD0.h"
#include "protoD0.h"
#include "uartD0.h"
#include "LCD_T6963.H"

char c_txstrD0[256];
char c_temp[20];

extern char pattern[32];

//*****************************************************************************
//	Kommand - Kommandointerpreter
//  Input:	Befehlsstring
//	Output:	-
//*****************************************************************************
void KommandointerpreterD0(char * pc_str)
{
unsigned char uc_ok = 0;

   if (strncmp(pc_str,"C",1) == 0)    uc_ok = BEFEHL_CLEAR(pc_str+1);
   if (strncmp(pc_str,"X",1) == 0)    uc_ok = BEFEHL_ARRAY_X(pc_str+1);
   if (strncmp(pc_str,"D:",2) == 0)   uc_ok = BEFEHL_ARRAY_1(pc_str+2);
   if (strncmp(pc_str,"E:",2) == 0)   uc_ok = BEFEHL_ARRAY_2(pc_str+2);
   if (strncmp(pc_str,"F:",2) == 0)   uc_ok = BEFEHL_ARRAY_3(pc_str+2);
   if (strncmp(pc_str,"G:",2) == 0)   uc_ok = BEFEHL_ARRAY_4(pc_str+2);

   if (strncmp(pc_str,"L:",2) == 0)   uc_ok = BEFEHL_LOAD(pc_str+2);
   if (strncmp(pc_str,"S",1) == 0)    uc_ok = BEFEHL_SET(pc_str+1);
   if (strncmp(pc_str,"K:",2) == 0)   uc_ok = BEFEHL_KEY(pc_str+2);
   if (strncmp(pc_str,"Q:",2) == 0)   uc_ok = BEFEHL_QKEY(pc_str+2);
   if (strncmp(pc_str,"Z:",2) == 0)   uc_ok = BEFEHL_STRING(pc_str+2);
   
   if (!uc_ok) SENDE_ERRORD0();
}

//*****************************************************************************
//  Kommand - Sende String an PC
//  Input:        -
//  Output:        -
//*****************************************************************************
void SENDE_ERRORD0(void)
{
  strcpy(c_txstrD0,"ERROR");
  sende_UARTD0(c_txstrD0);
}


//*****************************************************************************
//  Kommand - Halfbyte
//  Input:  0..9,A..F
//  Output: byte
//*****************************************************************************
unsigned char Halfbyte(unsigned char uc_x)
{
unsigned char uc_tmp;

   uc_tmp = uc_x;
   if(uc_tmp > 0x39) uc_tmp = uc_tmp - 0x37; else uc_tmp = uc_tmp - 0x30;
   return(uc_tmp);
}


//*****************************************************************************
//	Kommand - Kommando CLEAR
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_CLEAR(char * pc_str)
{
unsigned int ui_cnt;

  for(ui_cnt = 0; ui_cnt < 32; ui_cnt++)
     pattern[ui_cnt] = 0;

  for(ui_cnt = 0; ui_cnt < 4; ui_cnt++)
    LCD_CREATECHAR(ui_cnt+1, pattern + ui_cnt*8);

  LCD_CURSORPOS(1,5);
  LCD_WRITECHAR(1,5,0x20);
  LCD_WRITECHAR(2,5,0x20);
  LCD_WRITECHAR(1,6,0x20);
  LCD_WRITECHAR(2,6,0x20);

  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando ARRAY
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_ARRAY_X(char * pc_str)
{
unsigned int ui_cnt;
 
  for(ui_cnt = 0; ui_cnt < 4; ui_cnt++)
   	 LCD_CREATECHAR(ui_cnt+1, pattern + ui_cnt*8);

  LCD_CURSORPOS(1,5);
  LCD_WRITECHAR(1,5,0xA0);
  LCD_WRITECHAR(2,5,0xA1);
  LCD_WRITECHAR(1,6,0xA2);
  LCD_WRITECHAR(2,6,0xA3);

  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando ARRAY
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_ARRAY_1(char * pc_str)
{
unsigned int ui_cnt,ui_x;
unsigned int ui_offset = 0; //0..3
 
  ui_cnt = 0;
  while(ui_cnt <= strlen(pc_str))
  {
     ui_x = (Halfbyte(pc_str[ui_cnt])) << 4;
	 ui_cnt++;
     ui_x |= Halfbyte(pc_str[ui_cnt]);
	 ui_cnt++;

	 pattern[(ui_cnt >> 1) + (ui_offset *8) -1] = ui_x;
	 
	 if (ui_cnt >= 16) ui_cnt= 255; //Abbruch
  }
 
  LCD_CREATECHAR(ui_offset+1, pattern + ui_offset * 8);

  LCD_CURSORPOS(1,5);
  LCD_WRITECHAR(1,5,0xA0);
  LCD_WRITECHAR(2,5,0xA1);
  LCD_WRITECHAR(1,6,0xA2);
  LCD_WRITECHAR(2,6,0xA3);

  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando ARRAY
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_ARRAY_2(char * pc_str)
{
unsigned int ui_cnt,ui_x;
unsigned int ui_offset = 1; //0..3
 
  ui_cnt = 0;
  while(ui_cnt <= strlen(pc_str))
  {
     ui_x = (Halfbyte(pc_str[ui_cnt])) << 4;
	 ui_cnt++;
     ui_x |= Halfbyte(pc_str[ui_cnt]);
	 ui_cnt++;

	 pattern[(ui_cnt >> 1) + (ui_offset *8) -1] = ui_x;

	 if (ui_cnt >= 16) ui_cnt= 255; //Abbruch
  }
 
 
  LCD_CREATECHAR(ui_offset+1, pattern + ui_offset * 8);

  LCD_CURSORPOS(1,5);
  LCD_WRITECHAR(1,5,0xA0);
  LCD_WRITECHAR(2,5,0xA1);
  LCD_WRITECHAR(1,6,0xA2);
  LCD_WRITECHAR(2,6,0xA3);

  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando ARRAY
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_ARRAY_3(char * pc_str)
{
unsigned int ui_cnt,ui_x;
unsigned int ui_offset = 2; //0..3
 
  ui_cnt = 0;
  while(ui_cnt <= strlen(pc_str))
  {
     ui_x = (Halfbyte(pc_str[ui_cnt])) << 4;
	 ui_cnt++;
     ui_x |= Halfbyte(pc_str[ui_cnt]);
	 ui_cnt++;

	 pattern[(ui_cnt >> 1) + (ui_offset *8) -1] = ui_x;

	 if (ui_cnt >= 16) ui_cnt= 255; //Abbruch
  }
 
 
  LCD_CREATECHAR(ui_offset+1, pattern + ui_offset * 8);

  LCD_CURSORPOS(1,5);
  LCD_WRITECHAR(1,5,0xA0);
  LCD_WRITECHAR(2,5,0xA1);
  LCD_WRITECHAR(1,6,0xA2);
  LCD_WRITECHAR(2,6,0xA3);

  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando ARRAY
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_ARRAY_4(char * pc_str)
{
unsigned int ui_cnt,ui_x;
unsigned int ui_offset = 3; //0..3
 
  ui_cnt = 0;
  while(ui_cnt <= strlen(pc_str))
  {
     ui_x = (Halfbyte(pc_str[ui_cnt])) << 4;
	 ui_cnt++;
     ui_x |= Halfbyte(pc_str[ui_cnt]);
	 ui_cnt++;

	 pattern[(ui_cnt >> 1) + (ui_offset *8) -1] = ui_x;

	 if (ui_cnt >= 16) ui_cnt= 255; //Abbruch
  }
 
 
  LCD_CREATECHAR(ui_offset+1, pattern + ui_offset * 8);

  LCD_CURSORPOS(1,5);
  LCD_WRITECHAR(1,5,0xA0);
  LCD_WRITECHAR(2,5,0xA1);
  LCD_WRITECHAR(1,6,0xA2);
  LCD_WRITECHAR(2,6,0xA3);

  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando SET
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_SET(char * pc_str)
{
  LCD_CREATEARTEFACT();
	
  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);	
}


//*****************************************************************************
//	Kommand - Kommando LOAD
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_LOAD(char * pc_str)
{
char c_str[10];	
unsigned char aui_zahl[4];	
unsigned int ui_cnt,ui_x;
char * pc_s;

  for(ui_cnt = 0; ui_cnt < 4; ui_cnt++)
     aui_zahl[ui_cnt] = 0;

  ui_cnt = 0;
  pc_s = strtok (pc_str, ",");
  while(pc_s)
  {
	 ui_x = atoi(pc_s);
     if (ui_x > 32) aui_zahl[ui_cnt] = 0; else aui_zahl[ui_cnt] = ui_x;
	 ui_cnt++;  
	 pc_str += strlen(pc_s) +1;
	 pc_s = strtok (pc_str, ",");
	 if (!pc_s) pc_s = strtok (pc_str, "\r"); 
	 if (ui_cnt >= 4) pc_s = NULL;
  }
     
  LCD_CURSORPOS(11,3);
  LCD_CLEARLINE(3);
  for(ui_cnt=0;ui_cnt < 4;ui_cnt++)
  {
	LCD_WRITESTR(11+ui_cnt +ui_cnt*1,3,itoa(aui_zahl[ui_cnt],c_str,10));
  }
  
  if (aui_zahl[0] > 0) LCD_WRITEARTEFACT(11,5,aui_zahl[0]); else LCD_WRITECHAR(11,5,0x20);
  if (aui_zahl[1] > 0) LCD_WRITEARTEFACT(12,5,aui_zahl[1]); else LCD_WRITECHAR(12,5,0x20);
  if (aui_zahl[2] > 0) LCD_WRITEARTEFACT(11,6,aui_zahl[2]); else LCD_WRITECHAR(11,6,0x20);
  if (aui_zahl[3] > 0) LCD_WRITEARTEFACT(12,6,aui_zahl[3]); else LCD_WRITECHAR(12,6,0x20);
  
 
  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando KEY
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_KEY(char * pc_str)
{
unsigned int ui_x;

  ui_x = atoi(pc_str);
  LCD_CURSORPOS(1,5);
//  LCD_WRITECHAR(1,5,ui_x %256);
  LCD_WRITEARTEFACT(1,5,ui_x);
  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando Q
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_QKEY(char * pc_str)
{
  LCD_CURSORPOS(1,7);
  LCD_WRITEBIGCHAR(1,7,pc_str[0]);
  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando Z
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_STRING(char * pc_str)
{
  LCD_CURSORPOS(1,7);
  LCD_WRITEBIGSTR(1,7,pc_str);
  strcpy(c_txstrD0,"OK");
  sende_UARTD0(c_txstrD0);
  return(1);
}