//*****************************************************************************
//
//	UART D0 des ATXMega Interrupt gesteuert
//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdlib.h>
#include "uartD0.h"

#define BUFD0_LAENGE  256
#define USARTD0_TXINT 0x01  	
#define USARTD0_RXINT 0x10  	


char c_RXBUFD0[BUFD0_LAENGE+1];
char c_TXBUFD0[BUFD0_LAENGE+1];
  
unsigned int ui_RXD0cnt,ui_TXD0cnt,ui_RXD0wr,ui_TXD0wr,ui_RXD0rd,ui_TXD0rd;


//*****************************************************************************
//	RX - INT
//*****************************************************************************
ISR(USARTD0_RXC_vect)
{
  c_RXBUFD0[ui_RXD0wr]  = USARTD0.DATA;
  ui_RXD0wr++;
  
  if (ui_RXD0wr > BUFD0_LAENGE) ui_RXD0wr = 0;
  ui_RXD0cnt++; 
}

//*****************************************************************************
//	TX - INT
//*****************************************************************************
ISR(USARTD0_DRE_vect)                 //TX-Interrupt
{
  if (ui_TXD0cnt)
   {
     USARTD0.DATA = c_TXBUFD0[ui_TXD0rd];
	 ui_TXD0rd++;
	 if (ui_TXD0rd > BUFD0_LAENGE) ui_TXD0rd = 0;
	 if (ui_TXD0cnt > 0) ui_TXD0cnt--; 
   }
   else USARTD0.CTRLA &= 0xFC;  //TX-INT disable
}


//*****************************************************************************
//	UART - Initialisierung
//	Input:	-
//	Output:	-
//*****************************************************************************
void UARTD0_INIT(void)
{
unsigned int ui_i;

   cli();				//global INT aus
	                   
   ui_RXD0cnt = 0;
   ui_TXD0cnt = 0;
   ui_RXD0wr  = 0;
   ui_TXD0wr  = 0;
   ui_RXD0rd  = 0;
   ui_TXD0rd  = 0;
	
   for(ui_i=0;ui_i<BUFD0_LAENGE;ui_i++)
   {
	  c_TXBUFD0[ui_i] = 0;
	  c_RXBUFD0[ui_i] = 0;
   }
	
   PORTD.DIR = (PORTD.DIR & 0xFB) | 0x08; //PD3 TXD, PD2 RXD
   PORTD.OUT |= 0x0C;

   USARTD0.CTRLA = USARTD0_RXINT;  //Int
   USARTD0.CTRLB = 0x18;  //RX, TX Enable
   USARTD0.CTRLC = 0x03;  //8N1

   //2 MHz Clk  9,6kBd  -> BSel=1539, BScale=-7
//   USARTD0.BAUDCTRLA =  0x03;
//   USARTD0.BAUDCTRLB =  0x96;

   //2 MHz Clk  19,2kBd  -> BSel=705, BScale=-7
//   USARTD0.BAUDCTRLA =  0xC1;
//   USARTD0.BAUDCTRLB =  0x92;

   //2 MHz Clk  57,6kBd  -> BSel=150, BScale=-7
//   USARTD0.BAUDCTRLA =  0x96;
//   USARTD0.BAUDCTRLB =  0x90;

   //2 MHz Clk  115,2kBd  -> BSel=11, BScale=-7
//   USARTD0.BAUDCTRLA =  0x0B;
//   USARTD0.BAUDCTRLB =  0x90;

   //32 MHz Clk  9,6kBd  -> BSel=3317, BScale=-4
//   USARTD0.BAUDCTRLA =  0xF5;
//   USARTD0.BAUDCTRLB =  0xCC;

   //32 MHz Clk  19,2kBd  -> BSel=3301, BScale=-5
//   USARTD0.BAUDCTRLA =  0xE5;
//   USARTD0.BAUDCTRLB =  0xBC;

   //32 MHz Clk  57,6kBd  -> BSel=2158, BScale=-6
//   USARTD0.BAUDCTRLA =  0x6E;
//   USARTD0.BAUDCTRLB =  0xA8;

   //32 MHz Clk  115,2kBd  -> BSel=1047, BScale=-6
   USARTD0.BAUDCTRLA =  0x17;
   USARTD0.BAUDCTRLB =  0xA4;
	
   USARTD0.DATA;

   CCP = CCP_IOREG_gc; //CPU Change Protection
   PMIC.CTRL = 0x07;	//Int enable

   sei();				//global INT an
}
 
//*****************************************************************************
//	UART - Test, ob Zeichen empfangen
//	Input:	-
//	Output:	1 ja , 0 nein
//*****************************************************************************
unsigned char UARTD0_RXTest(void)
{
   if (ui_RXD0cnt) return(1); 
        	  else return(0);
}


//*****************************************************************************
//	UART - Zeichen senden
//	Input:	char  Zeichen
//	Output:	-
//*****************************************************************************
void UARTD0_TX(char c_x)
{
//  while(ui_TXD0cnt > 0); //warte bis Puffer leer

   c_TXBUFD0[ui_TXD0wr] = c_x;
   ui_TXD0wr++;
   if (ui_TXD0wr > BUFD0_LAENGE) ui_TXD0wr = 0;
   ui_TXD0cnt++; 
   if (ui_TXD0cnt > BUFD0_LAENGE) ui_TXD0cnt = 0; 
 
   USARTD0.CTRLA |= USARTD0_TXINT;	//TX-Int enable  	
}


//*****************************************************************************
//	UART - String senden
//	Input:	char* String 
//	Output:	-
//*****************************************************************************
void UARTD0_TXStr(char * pc_str)
{
   while(ui_TXD0cnt > 0); //warte bis Puffer leer

   while (*pc_str != '\0')
   {
      c_TXBUFD0[ui_TXD0wr] = *pc_str;
      ui_TXD0wr++;
      if (ui_TXD0wr > BUFD0_LAENGE) ui_TXD0wr = 0;
      ui_TXD0cnt++; 
      if (ui_TXD0cnt > BUFD0_LAENGE) ui_TXD0cnt = 0;
	  pc_str++; 
   }	 
 
   USARTD0.CTRLA |= USARTD0_TXINT;	//TX-Int enable  	
}


//*****************************************************************************
//	UART - Zeichen empfangen
//	Input:	- 
//	Output:	char  Empfangszeichen
//*****************************************************************************
char UARTD0_RX(void)
{      
char uc_x;
         
  while(ui_RXD0cnt == 0);		//warte auf Zeichen

  uc_x = c_RXBUFD0[ui_RXD0rd];
  
  cli();  //INTs sperren  
  ui_RXD0rd++;
  if (ui_RXD0rd > BUFD0_LAENGE) ui_RXD0rd = 0;
  if (ui_RXD0cnt > 0) ui_RXD0cnt--;  
  sei();  //INTs freigeben  
  
  return(uc_x);
}

